
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.lcdui.game.LayerManager;
import javax.microedition.lcdui.game.Sprite;
import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotOpenException;

import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/**
 * Entre 2 Almas	[https://sourceforge.net/projects/entre2almas/]  
 * ------------------------------------------------------------------------
 * This file is part of "Entre 2 Almas".
 * "Entre 2 Almas" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * "Entre 2 Almas" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "Entre 2 Almas"; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ------------------------------------------------------------------------
 * 		Copyright 2005 - 2006 Pedro Javier Sez Martnez
 * 
 * 
 * Lleva toda la direccin de lo que es el ncleo del juego, aqu se decide
 * cuando mirar colisiones, cuando ejecutar acciones, como buscar una Ruta,
 * cuales son las funciones de cada tecla en cada momento, etc.
 * 
 * @author Pedro Javier Sez Martnez
 * @version 1.0
 */
public class Ejecucion extends GameCanvas implements Runnable {

	/**
	 * Referencia al midlet.
	 */
	private Entre2Almas midlet;

	/**
	 * Hilo de ejecucin del juego
	 */
	Thread t;

	/**
	 * Ser el controlador de las distintas capas del juego.
	 */
	private LayerManager LM;

	/**
	 * Parser del XML.
	 */
	KXmlParser parser;

	/**
	 * El pathFinding A*
	 */
	PathFinding buscaRuta;

	/**
	 * Estar a true cuando Oscar est caminando.
	 */
	private boolean caminando;

	/**
	 * Si es 0, es que va de frente, 1 va de espaldas, 2 derecha, 3 izquierda.
	 */
	private int direccion;

	/**
	 * Indice del array Path que contiene la posicin del siguiente paso a
	 * ejecutar por oscar.
	 */
	private int pasoActual;

	/**
	 * Son los distintos mapas del juego
	 */
	AVLTree mapas;

	/**
	 * Indica el identificador del mapa actual;
	 */
	public int mapaActual;

	/**
	 * El ancho y el alto en pxeles que tiene cada celda en las que esta
	 * dividido el mapa
	 */
	private int anchoCelda, altoCelda;

	/**
	 * Escenario que actualmente est cargado en memoria.
	 */
	public Escenario escenActual;

	/**
	 * Guarda el nombre del ltimo escenario cargado
	 */
	private StringBuffer ultimoEscenario;

	/**
	 * Sprite general
	 */
	private Sprite sprite;

	/**
	 * Sprite de Oscar
	 */
	private Sprite oscar;

	/**
	 * Sprite del puntero.
	 */
	private Sprite puntero;

	/**
	 * Sprite formado por la imagen correspondiente a la parte de atrs del
	 * escenario actual.
	 */
	private Sprite escenDetras;

	/**
	 * Sprite formado por la imagen correspondiente a la parte caminable del
	 * escenario actual.
	 */
	private Sprite escenCaminable;

	/**
	 * Vector que almacena los Items que tenemos en el inventario
	 */
	private Vector itemsInventario;

	/**
	 * Indica si hay que mostrar o no,el inventario
	 */
	private boolean mostrarInventario;

	/**
	 * Indica el objeto que est mostrndose en ese momento en el inventario.
	 */
	private int invSelec;

	/**
	 * Indica el Seleccionable que esta siendo seleccionado en la pantalla de
	 * juego
	 */
	private Opcionable seleccionado;

	/**
	 * Indica si hay que mostrar o no, las opciones de un objeto
	 */
	private boolean mostrarOpciones;

	/**
	 * Indica si el puntero ha colisionado con algo.
	 */
	private boolean colisionado;

	/**
	 * Marca el espacio de ancho que se deja para el inventario en la pantalla
	 * de juego
	 */
	private static int anchoInventario;

	/**
	 * Marca el espacio de alto que se deja para el inventario en la pantalla de
	 * juego
	 */
	private static int altoInventario;

	/**
	 * Ancho del panel donde se van a mostrar las opciones
	 */
	private static int anchoOpciones;

	/**
	 * Alto del panel donde se van a mostrar las opciones
	 */
	private static int altoOpciones;

	/**
	 * Ancho del panel donde se van a mostrar los mensajes
	 */
	private static int anchoMensaje;

	/**
	 * Alto del panel donde se van a mostrar los mensajes
	 */
	private static int altoMensaje;

	/**
	 * indica la opcin que est seleccionada ahora mismo para un Item.
	 */
	private int opcionSele;

	/**
	 * Indica si hay que mostrar o un mensaje.
	 */
	private boolean mostrarMensaje;

	/**
	 * Vector con los videos correspondientes a una fase
	 */
	private AVLTree videosFase;

	/**
	 * Identificador del siguiente video a mostrar.
	 */
	private int videoSiguiente;

	/**
	 * AVL con las conversaciones correspondientes a una fase
	 */
	private AVLTree converFase;

	/**
	 * Identificador de la conversacin a mostrar
	 */
	private int converSelec;

	/**
	 * Participante en la conversacin.
	 */
	private Item participante;

	/**
	 * Indica la longitud que tendr el id de la opcin a mostrar en la
	 * conversacin
	 */
	private int longIdOp;

	/**
	 * Indica las letras que tendr que tener en su inicio el id de la opcin,
	 * para que entonces esta sea mostrada. En caso de que iniIdOp no contenga
	 * ninguna letra, se cargarn todas las opciones que tengan un id que
	 * coincida con longIdOp
	 */
	private StringBuffer iniIdOp;

	/**
	 * Indica si el que esta hablando es el participante (no Oscar),en cuyo caso
	 * estar a true.
	 */
	private boolean habParti;

	/**
	 * Indica si una tecla est pulsada en este momento.
	 */
	private boolean teclaPulsada;

	/**
	 * Numero de fase en la que nos encontramos
	 */
	public int nFase;

	/**
	 * Nombre de la fase en la que nos encontramos.
	 */
	private StringBuffer nomFase;

	/**
	 * Parte o pantalla de juego en la que nos encontramos. Las posibilidades
	 * son: Video, Juego, inventario, conversacin, minijuego1, minijuego2 o
	 * minijuego 3
	 */
	public int parteJuego;

	/**
	 * Variables estticas que representan los distintos tipos de pantalla que
	 * existen
	 */
	private final int video = 0;

	private final int juego = 1;

	private final int inventario = 2;

	private final int conversacion = 3;

	private final int mapa = 4;

	/**
	 * Indica que pantalla estamos viendo actualmente.
	 */
	public int actual;

	/**
	 * Tiempo de delays acumulado. En cada bucle del hilo se hace un delay por
	 * ejemplo de 20 milsg, pues en tmpAcumulado vamos sumando esto.
	 */
	private int tmpAcumulado;

	/**
	 * Indica que se est cargando una fase
	 */
	private boolean cargando;

	/**
	 * Vector que guardar los resultados a ejecutar a una determinada accin.
	 */
	Vector resultados;

	/**
	 * Indica cuando el programa est ocupado ejecutando una accin.
	 */
	private boolean ocupado;

	/**
	 * Indica cuando es necesario redibujar la pantalla.
	 */
	private boolean redibujar;

	/**
	 * Indica que queremos ir al men.
	 */
	private boolean irMenu;

	/**
	 * Posiciones X e Y en las que aparece el enlace directo al men
	 */
	private int posXMenu, posYMenu;

	/**
	 * Ancho y alto del acceso directo al men.
	 */
	private int anchoMenu, altoMenu;

	/**
	 * Indica que un item est preparado en este momento para ser combinado con
	 * otros items.
	 */
	private boolean combinando;

	/**
	 * Indica que el item que est preparado en este momento para ser combinado,
	 * ya est intentando ser combinado con otro item.
	 *  
	 */
	private boolean intentaCombinar;

	/**
	 * Item que est siendo combinando
	 */
	private Item combinado;

	/**
	 * Nos indicara cuando salir del bucle de ejecucin, y por tanto finalizar
	 * el hilo
	 */
	boolean salir;

	/**
	 * Es la letra inicial desde la que hay que empezar mostrar el mensaje en la
	 * lnea de texto. Por defecto su valor ser -6. El porque es para que
	 * tarden ms en empezar a moverse las letras.
	 */
	int letra;

	/**
	 * Espacio disponible en pxeles en la lnea de texto
	 */
	int disponible;

	/**
	 * Numero de letras que caben en la lnea de texto
	 */
	int numLetras;

	/**
	 * COnstructor de la clase Ejecucin
	 * 
	 * @param midlet
	 *            El midlet del juego. Entre2Almas
	 */
	public Ejecucion(Entre2Almas midlet) {
		super(true);
		this.midlet = midlet;
		//		El orden en que se ven las capas es desde la ultima a la
		//primera, es decir si lo primero que metes es una imagen
		//de un lobo, es lobo estar siempre en primera plana
		LM = new LayerManager();
		itemsInventario = new Vector();
		mapas = new AVLTree();
		mapaActual = 0;
		//Ponemos la pantalla a modo completo
		setFullScreenMode(true);
		//inicializamos el Vector
		videosFase = new AVLTree();
		converFase = new AVLTree();
		//Cargamos el tamao de la pantalla y el punto de inicio donde se
		//van a poner las imgenes.
		Utils.width = getWidth();
		Utils.height = getHeight();
		Utils.inicioPantallaX = (Utils.width / 2) - (Utils.getTamGrupoX() / 2);
		Utils.inicioPantallaY = (Utils.height / 2) - (Utils.getTamGrupoY() / 2);
		//		Cargamos el sprite del puntero.
		Utils.cargarImagen(Utils.spPuntero);
		puntero = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		puntero.setFrameSequence(Utils.getFrames(Utils.spPuntero));
		//Situamos el puntero en el medio.
		puntero.setPosition(Utils.inicioPantallaX + (Utils.getTamGrupoX() / 2),
				Utils.inicioPantallaY + (Utils.getTamGrupoY() / 2));
		//Cargamos el Sprite de Oscar.
		Utils.cargarImagen(Utils.spOscarFren);
		oscar = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		oscar.setFrameSequence(Utils.getFrames(Utils.spOscarFren));
		if (Utils.grupo == 0) {//grupo pequeo
			altoInventario = 42;
			anchoOpciones = Utils.lowFont.charWidth('a') * 8;
		} else {//grupo grande
			altoInventario = 66;
			anchoOpciones = Utils.highFont.charWidth('a') * 8;
		}
		//			Establecemos el ancho y alto de cada una de las celdas
		anchoCelda = 2;
		altoCelda = 2;
		anchoInventario = Utils.width - anchoOpciones - Utils.inicioPantallaX
				- (Utils.width - Utils.inicioPantallaX - Utils.getTamGrupoX());
		anchoMensaje = Utils.getTamGrupoX() * 80 / 100;
		altoMensaje = (Utils.getTamGrupoY() - Utils.getSpaceTextItem() - altoInventario) * 90 / 100;
		teclaPulsada = false;
		colisionado = false;
		mostrarOpciones = false;
		opcionSele = 0;
		mostrarInventario = false;
		invSelec = 0;
		converSelec = 0;
		longIdOp = 1;
		iniIdOp = new StringBuffer();
		habParti = false;
		mostrarMensaje = false;
		cargando = false;
		resultados = new Vector();
		ocupado = false;
		tmpAcumulado = 0;
		pasoActual = 0;
		direccion = 0;
		combinando = false;
		intentaCombinar = false;
		//le pongo -6 para que tarde ms en empezar a moverse las letras.
		letra = -6;
		ultimoEscenario = new StringBuffer();
		ultimoEscenario.append("");
		//Se empieza por la fase 1
		actual = -1;
		nFase = 1;
		nomFase = new StringBuffer();
		//Se empieza mostrando el video inicial
		videoSiguiente = 0;
		parteJuego = video;
		redibujar = true;
		irMenu = false;
		//Medidas del acceso directo al men.
		if (Utils.grupo == 0) {
			posXMenu = Utils.inicioPantallaX + Utils.getTamGrupoX()
					- Utils.highFont.stringWidth("Menu") - 2;
			posYMenu = Utils.inicioPantallaY + Utils.getTamGrupoY()
					- Utils.getSpaceTextItem();
			anchoMenu = Utils.highFont.stringWidth("Menu") + 2;
			altoMenu = Utils.getSpaceTextItem();
		} else {
			posXMenu = Utils.inicioPantallaX + Utils.getTamGrupoX()
					- Utils.highFont.stringWidth("Menu") - 2;
			posYMenu = Utils.inicioPantallaY + Utils.getTamGrupoY()
					- Utils.getSpaceTextItem();
			anchoMenu = Utils.highFont.stringWidth("Menu") + 2;
			altoMenu = Utils.getSpaceTextItem();
		}
		disponible = Utils.getTamGrupoX() - anchoMenu;
		numLetras = disponible / Utils.widthLF;
	}

	public void start() {
		//crea el hilo y lo inicia (al iniciarlo se ejecuta el mtodo run()
		t = new Thread(this);
		t.start();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			Graphics g = getGraphics();
			//			lo pongo aqu porque si lo creo en ejecucin() en el nokia 6230
			// me
			//da outOfMemory, debido a que se junta lo que consume de memoria
			//la clase PathFinding (debido a sus numerosos arrays) y lo que
			// consumen
			//de memoria el cargar una fase.
			int numCeldasX = Utils.getTamGrupoX() / anchoCelda;
			int numCeldasY = Utils.getTamGrupoY() / altoCelda;
			buscaRuta = new PathFinding(numCeldasX, numCeldasY);
			//En caso de que estemos cargando una partida.
			ByteArrayInputStream bais = null;
			DataInputStream dis = null;
			byte[] datosEsen = null;
			if (Utils.cargandoPartida) {
				datosEsen = midlet.cargaDatosEsenciales();
				bais = new ByteArrayInputStream(datosEsen);
				dis = new DataInputStream(bais);
				nFase = dis.readInt();
			}
			cargarFase(g, nFase);

			if (Utils.cargandoPartida) {
				mapaActual = dis.readInt();
				((Mapa) mapas.search(mapaActual).data).activa = dis.readInt();
				((Mapa) mapas.search(mapaActual).data).getEstanActual().actual = dis
						.readInt();
				parteJuego = dis.readInt();

				bais.close();
				dis.close();

				escenActual = ((Mapa) mapas.search(mapaActual).data)
						.getEscenActual();

				//Una vez cargada la fase e introducidos los datos esenciales,
				//introducimos los Resultados a ejecutar
				midlet.cargarResultados();
				parser = null;
				System.gc();
			}

			salir = false;
			while (!salir) {
				synchronized (this) {
					if (Utils.guardandoPartida) {
						guardarPartida(g);
					}
					System.gc();
					//Recogemos las pulsaciones de teclado.
					input();
					switch (parteJuego) {
					case video:

						break;
					case juego:
						animarItems();
						if (caminando)
							avanzarPaso();
						if (resultados.size() > 0 && !ocupado)
							ejecutarResultado();
						break;
					case conversacion:
						if (resultados.size() > 0 && !ocupado)
							ejecutarResultado();
						if (resultados.size() == 0 && !ocupado && !mostrarOpciones) {
							mostrarOpciones = true;
							Utils.startWidth = Utils.inicioPantallaX
									+ ((oscar.getWidth() * 32) / 100);
							Utils.spaceMen = Utils.getTamGrupoX()
									- ((oscar.getWidth() * 60) / 100);
							redibujar = true;
						}
						moverBoca();
						break;
					case mapa:
						if (resultados.size() > 0 && !ocupado)
							ejecutarResultado();
						break;
					}
					//Redibujamos la pantalla
					if (redibujar)
						drawScreen(g);
					// El delay especifica el tiempo que se dormir el hilo
					// entre
					// ejecucin y ejecucin.
					//si pusisemos un delay menor el juego ira mas rpido
					tmpAcumulado = tmpAcumulado + 20;
					if (tmpAcumulado > 20000)
						tmpAcumulado = 0;
					Thread.sleep(20);
				}
			}
			salir();
		} catch (InterruptedException ie) {
			System.err.println("Problemas con el Thread\n" + ie);
			midlet.mostrarAlerta("Ejecucion::run::Problemas con el Thread\n"
					+ ie, AlertType.ERROR);
			ie.printStackTrace();
		} catch (NullPointerException ie) {
			System.err
					.println("Ejecucion::run::Problemas con un null\n"
							+ ie);
			midlet
					.mostrarAlerta(
							"Ejecucion::run::Problemas con un null\n"+ ie,
							AlertType.ERROR);
			
			ie.printStackTrace();
		} catch (XmlPullParserException e) {
			System.err.println("Problemas con el XML\n" + e);
			midlet.mostrarAlerta("Ejecucion::run::Problemas con el XML\n" + e,
					AlertType.ERROR);
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("Problemas de entrada/salida\n" + e);
			midlet.mostrarAlerta(
					"Ejecucion::run::Problemas de entrada/salida\n" + e,
					AlertType.ERROR);
			e.printStackTrace();
		} catch (Exception e) {
			System.err.println("Problemas desconocido\n" + e);
			midlet.mostrarAlerta("Ejecucion::run::Problemas desconocido\n" + e,
					AlertType.ERROR);
			e.printStackTrace();
		}
	}

	/**
	 * KeyPressed es llamado cuando una tecla es pulsada, pero solo lo usaremos
	 * cuando sea una non-game Key. Las teclas de numeros (1,2,3,...), el * y la # ,
	 * son game Keys y tienen positivos KeyCodes. As las non-game keys tienen
	 * keyCodes negativos. Cuando se pulse una de estas teclas especiales,
	 * iremos al men.
	 * 
	 * @param keyCode
	 *            keyCode de la tecla pulsada.
	 */
	public void keyPressed(int keyCode) {
		if (keyCode < 0) {
			irMenu = true;
		}
	}

	/**
	 * mtodo para controlar las entradas de teclado
	 * 
	 * @throws InterruptedException
	 */
	private void input() throws InterruptedException {
		//Recogemos el estado de las teclas
		int keyStates = getKeyStates();
		if (irMenu)
			keyStates = GameCanvas.GAME_D_PRESSED;
		switch (parteJuego) {
		case video:
			if (((keyStates & UP_PRESSED) != 0) && Utils.linea > 0
					&& !teclaPulsada) {
				Utils.linea--;
			} else if ((((keyStates & DOWN_PRESSED) != 0))
					&& Utils.numLineasMensaje - Utils.linea > Utils.numLineas
					&& !teclaPulsada) {
				Utils.linea++;
			} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
				Video video = (Video) videosFase.search(videoSiguiente).data;
				//					Comprobacin para que no de vueltas y vueltas en los mismo
				//frames
				if (sprite.getFrame() == Utils.getNumFrames(video.vinyeta) - 1) {
					parteJuego = video.sigParte;
					ocupado = false;
				} else {
					//Preparamos la siguiente vieta del video
					sprite.nextFrame();
					Utils.linea = 0;
				}
			}
			break;
		case juego:
			if (!mostrarOpciones && !mostrarInventario && !mostrarMensaje) {//Si
				// no estamos mostrando las  opciones de ,un ,objeto...
				//lo que est comentado son ANDs que se podran poner en
				//caso de que se quisiese limitar, por razones de ejecucin,
				//las veces que se entra en cada uno de los ifs
				if (((keyStates & UP_PRESSED) != 0) /*
													 * &&
													 * puntero.getY()>Utils.inicioPantallaY
													 */) {
					puntero.setPosition(puntero.getX(), Math.max(
							Utils.inicioPantallaY, puntero.getY() - 2
									- Utils.grupo));
					redibujar = true;
				} else if ((((keyStates & DOWN_PRESSED) != 0))/*
															   * &&puntero.getY()+puntero.getHeight()
															   * <Utils.inicioPantallaY+Utils.tamGrupoY()
															   */) {
					if (puntero.getX() + puntero.getWidth() > posXMenu)
						puntero.setPosition(puntero.getX(), Math.min(
								Utils.inicioPantallaY + Utils.getTamGrupoY()
										- puntero.getHeight() - altoMenu,
								puntero.getY() + 2 + Utils.grupo));
					else
						puntero.setPosition(puntero.getX(), Math.min(
								Utils.inicioPantallaY + Utils.getTamGrupoY()
										- puntero.getHeight(), puntero.getY()
										+ 2 + Utils.grupo));
					redibujar = true;
				} else if ((keyStates & LEFT_PRESSED) != 0 /*
														    * &&
														    * puntero.getX()>Utils.inicioPantallaX
														    */) {
					puntero.setPosition(Math.max(Utils.inicioPantallaX, puntero
							.getX()
							- 2 - Utils.grupo), puntero.getY());
					redibujar = true;
				} else if ((keyStates & RIGHT_PRESSED) != 0 /*
															 * &&
															 * puntero.getX()+puntero.getWidth()
															 * <Utils.inicioPantallaX+Utils.tamGrupoX()
															 */) {
					if (puntero.getY() + puntero.getHeight() > posYMenu)
						puntero.setPosition(Math.min(Utils.inicioPantallaX
								+ Utils.getTamGrupoX() - puntero.getWidth()
								- anchoMenu, puntero.getX() + 2 + Utils.grupo),
								puntero.getY());
					else
						puntero.setPosition(Math.min(Utils.inicioPantallaX
								+ Utils.getTamGrupoX() - puntero.getWidth(),
								puntero.getX() + 2 + Utils.grupo), puntero
								.getY());
					redibujar = true;
				} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
					if (colisionado) {
						if (combinando) {
							conseCombinacion();
							intentaCombinar = true;
						}
						//Si est ocupado no se podrn mostrar las opciones
						//de un objeto, debido al tema de que al cambiar de
						//escenario podra cascar.
						else {
							if (seleccionado != null && !ocupado)
								mostrarOpciones = true;
						}
						/*
						 * else podra ser nuestro personaje principal y aqu
						 * por ejemplo hacer que muestre un mensaje por pantalla
						 */
					} else {
						if (buscarRuta())
							caminando = true;
					}
				} else if (((keyStates & GameCanvas.GAME_C_PRESSED) != 0)
						&& !teclaPulsada) {
					mostrarInventario = true;
					if (colisionado) {
						colisionado = false;
						letra = -6;
						puntero.nextFrame();
						seleccionado = null;
					}
				}
				//Comprobamos si hay colisin, solo cuando la ltima pulsacin
				// de tecla no sirvi ni para entrar en inventario ni para
				// entrar en las opciones.
				//Es decir que sirvi para mover el puntero.
				//Y solo lo comprobamos cuando no est ocupado ejecutando un
				//resultado, ya que hay algunos resultados que necesitan que
				//el objeto con el colisiona el puntero, no cambie.
				if (resultados.size() == 0) {
					if (!colisionado && !mostrarInventario && !mostrarOpciones
							&& teclaPulsada) {
						if (colisionPunItem()) {
							colisionado = true;
							puntero.nextFrame();
						}
					} else if (colisionado && !mostrarInventario
							&& !mostrarOpciones && teclaPulsada) {
						if (!colisionPunItem()) {
							colisionado = false;
							letra = -6;
							seleccionado = null;
							System.gc();
							puntero.nextFrame();
						}
					}
				}
			} else if (mostrarOpciones) {//si estamos mostrando las opciones de
				// un objeto...
				if ((keyStates & UP_PRESSED) != 0 && !teclaPulsada) {
					if (opcionSele > 0)
						opcionSele--;
					else
						opcionSele = Utils.numOps - 1;
				} else if ((keyStates & DOWN_PRESSED) != 0 && !teclaPulsada) {
					if (opcionSele < Utils.numOps - 1)
						opcionSele++;
					else
						opcionSele = 0;
				} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
					mostrarOpciones = false;
					opcionesIO();
				}
			} else if (mostrarMensaje) {
				if (((keyStates & UP_PRESSED) != 0) && Utils.linea > 0
						&& !teclaPulsada) {
					Utils.linea--;
				} else if ((((keyStates & DOWN_PRESSED) != 0))
						&& Utils.numLineasMensaje - Utils.linea > Utils.numLineas
						&& !teclaPulsada) {
					Utils.linea++;
				} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
					mostrarMensaje = false;
					ocupado = false;
					Utils.linea = 0;
				}
			} else if (mostrarInventario) {
				if (((keyStates & LEFT_PRESSED) != 0) && !teclaPulsada) {
					if (itemsInventario.size() > 1) {
						if (invSelec > 0)
							invSelec--;
						else
							invSelec = itemsInventario.size() - 1;
						letra = -6;
					}
				} else if (((keyStates & RIGHT_PRESSED) != 0) && !teclaPulsada) {
					if (itemsInventario.size() > 1) {
						if (invSelec < itemsInventario.size() - 1)
							invSelec++;
						else
							invSelec = 0;
						letra = -6;
					}
				} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
					if (combinando) {
						conseCombinacion();
						intentaCombinar = true;
					}
					//Si est ocupado no se podrn mostrar las opciones
					//de un objeto, debido al tema de que al cambiar de
					//escenario podra cascar.
					else {
						if (itemsInventario.size() != 0)
							mostrarOpciones = true;
					}
				} else if (((keyStates & GameCanvas.GAME_C_PRESSED) != 0)
						&& !teclaPulsada) {
					mostrarInventario = false;
					invSelec = 0;
					letra = -6;
				}
			}

			break;
		case conversacion:
			Conversacion con = (Conversacion) seleccionado;
			if (((keyStates & UP_PRESSED) != 0) && Utils.linea > 0
					&& !teclaPulsada) {
				Utils.linea--;
			} else if ((((keyStates & DOWN_PRESSED) != 0))
					&& Utils.numLineasMensaje - Utils.linea > Utils.numLineas
					&& !teclaPulsada) {
				Utils.linea++;
			} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
				Utils.linea = 0;
				if (mostrarOpciones) {
					mostrarOpciones = false;
					Utils.startWidth = Utils.inicioPantallaX;
					Utils.spaceMen = Utils.getTamGrupoX();
					ocupado = true;
					opcionesIO();
				} else {
					//						Cierra la boca del que esta hablando, en caso de que
					// alguien hable
					cerrarBoca();
					if (habParti)
						habParti = false;
					if (ocupado)
						ocupado = false;
				}
			} else if (((keyStates & LEFT_PRESSED) != 0) && !teclaPulsada
					&& mostrarOpciones == true) {
				if (con.opciones.length != 0) {
					if (opcionSele > 0)
						opcionSele--;
					else
						opcionSele = con.opciones.length - 1;
					//						Ahora hayamos las opciones que pueden ser mostradas
					//salir se pondr a true cuando se cumplan las condiciones
					// de una opcin
					//NOTA: Esto no se puede quitar, ya que aunque en
					// getConversacion() hacemos
					//algo parecido, no es exactamente igual, ya que aquel vale
					//para el caso de pulsar accin o derecha (ya que siempre
					// hace
					//opcionSele++, mientras que este vale para izquierda,
					// siempre opcionSele--;
					boolean salir = false;
					while (!salir) {
						if (con.opciones[opcionSele] != null) {
							if (con.opciones[opcionSele].id.length() == longIdOp) {
								if (longIdOp == 1)
									salir = true;
								else {
									if ((con.opciones[opcionSele].id.substring(
											0, longIdOp - 1).equals(iniIdOp
											.toString())))
										salir = true;
									else {
										if (opcionSele > 0)
											opcionSele--;
										else
											opcionSele = con.opciones.length - 1;
									}
								}
							} else {
								if (opcionSele > 0)
									opcionSele--;
								else
									opcionSele = con.opciones.length - 1;
							}
						} else {
							if (opcionSele > 0)
								opcionSele--;
							else
								opcionSele = con.opciones.length - 1;
						}
						if (con.opciones[opcionSele] != null) {
							if (!seleccionado.getAccionActiva(opcionSele)
									&& salir == true) {
								if (opcionSele > 0)
									opcionSele--;
								else
									opcionSele = con.opciones.length - 1;
								salir = false;
							}
						}
					}
					Utils.linea = 0;
				}
			} else if (((keyStates & RIGHT_PRESSED) != 0) && !teclaPulsada
					&& mostrarOpciones == true) {
				if (con.opciones.length != 0) {
					if (opcionSele < con.opciones.length - 1)
						opcionSele++;
					else
						opcionSele = 0;
					Utils.linea = 0;
				}
			}
			break;
		case mapa:
			if (!mostrarOpciones && !mostrarMensaje) {//Si no estamos mostrando
				// las opciones de un
				// objeto...
				//lo que est comentado son ANDs que se podran poner en
				//caso de que se quisiese limitar, por razones de ejecucin,
				//las veces que se entra en cada uno de los ifs
				if (((keyStates & UP_PRESSED) != 0) /*
													 * &&
													 * puntero.getY()>Utils.inicioPantallaY
													 */) {
					puntero.setPosition(puntero.getX(), Math.max(
							Utils.inicioPantallaY, puntero.getY() - 2
									- Utils.grupo));
					redibujar = true;
				} else if ((((keyStates & DOWN_PRESSED) != 0))/*
															   * &&puntero.getY()+puntero.getHeight()
															   * <Utils.inicioPantallaY+Utils.tamGrupoY()
															   */) {
					if (puntero.getX() + puntero.getWidth() > posXMenu)
						puntero.setPosition(puntero.getX(), Math.min(
								Utils.inicioPantallaY + Utils.getTamGrupoY()
										- puntero.getHeight() - altoMenu,
								puntero.getY() + 2 + Utils.grupo));
					else
						puntero.setPosition(puntero.getX(), Math.min(
								Utils.inicioPantallaY + Utils.getTamGrupoY()
										- puntero.getHeight(), puntero.getY()
										+ 2 + Utils.grupo));
					redibujar = true;
				} else if ((keyStates & LEFT_PRESSED) != 0 /*
														    * &&
														    * puntero.getX()>Utils.inicioPantallaX
														    */) {
					puntero.setPosition(Math.max(Utils.inicioPantallaX, puntero
							.getX()
							- 2 - Utils.grupo), puntero.getY());
					redibujar = true;
				} else if ((keyStates & RIGHT_PRESSED) != 0 /*
															 * &&
															 * puntero.getX()+puntero.getWidth()
															 * <Utils.inicioPantallaX+Utils.tamGrupoX()
															 */) {
					if (puntero.getY() + puntero.getHeight() > posYMenu)
						puntero.setPosition(Math.min(Utils.inicioPantallaX
								+ Utils.getTamGrupoX() - puntero.getWidth()
								- anchoMenu, puntero.getX() + 2 + Utils.grupo),
								puntero.getY());
					else
						puntero.setPosition(Math.min(Utils.inicioPantallaX
								+ Utils.getTamGrupoX() - puntero.getWidth(),
								puntero.getX() + 2 + Utils.grupo), puntero.getY());
					
					redibujar = true;
				} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
					if (colisionado) {
						if (seleccionado != null)
							mostrarOpciones = true;
					}
				}
				//					Comprobamos si hay colisin, solo cuando la ltima pulsacin
				// de tecla no sirvi ni para entrar en inventario ni para
				// entrar en las opciones.
				//Es decir que sirvi para mover el puntero.
				if (!colisionado && !mostrarInventario && !mostrarOpciones
						&& teclaPulsada) {
					if (colisionPunEstan()) {
						colisionado = true;
						puntero.nextFrame();
					}
				} else if (colisionado && !mostrarInventario
						&& !mostrarOpciones && teclaPulsada) {
					if (!colisionPunEstan()) {
						colisionado = false;
						letra = -6;
						seleccionado = null;
						System.gc();
						puntero.nextFrame();
					}
				}
			} else if (mostrarOpciones) {//si estamos mostrando las opciones de
				// un objeto...
				if ((keyStates & UP_PRESSED) != 0 && !teclaPulsada) {
					if (opcionSele > 0)
						opcionSele--;
					else
						opcionSele = Utils.numOps - 1;
				} else if ((keyStates & DOWN_PRESSED) != 0 && !teclaPulsada) {
					if (opcionSele < Utils.numOps - 1)
						opcionSele++;
					else
						opcionSele = 0;
				} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
					mostrarOpciones = false;
					opcionesIO();
				}
			} else if (mostrarMensaje) {
				if (((keyStates & UP_PRESSED) != 0) && Utils.linea > 0
						&& !teclaPulsada) {
					Utils.linea--;
				} else if ((((keyStates & DOWN_PRESSED) != 0))
						&& Utils.numLineasMensaje - Utils.linea > Utils.numLineas
						&& !teclaPulsada) {
					Utils.linea++;
				} else if (((keyStates & FIRE_PRESSED) != 0) && !teclaPulsada) {
					mostrarMensaje = false;
					ocupado = false;
					Utils.linea = 0;
				}
			}
			break;
		}
		if ((keyStates & GameCanvas.GAME_D_PRESSED) != 0) {
			midlet.menu();
			this.wait();
		}
		if (keyStates != 0) {
			if (!teclaPulsada)
				redibujar = true;
			teclaPulsada = true;
		} else
			//KeyStates ser 0 cuando no haya ninguna tecla pulsada.
			teclaPulsada = false;
	}

	private void drawScreen(Graphics g) throws XmlPullParserException,
			IOException {
		if (cargando) {//si estamos cargando fase
			//pintamos la imagen de cargando
			sprite.paint(g);
			sprite.nextFrame();
			Font f = g.getFont();
			g.setFont(Utils.lowFont);
			if (Utils.grupo == 0)//grupo pequeo no pone suprimimos la primera
				// palabra Fase
				g.drawString(nFase + ": " + nomFase,
						(Utils.width - Utils.lowFont.stringWidth(nFase + ": "
								+ nomFase.toString())) / 2,
						Utils.inicioPantallaY, Graphics.TOP | Graphics.LEFT);
			else
				g.drawString("Fase " + nFase + ": " + nomFase,
						(Utils.width - Utils.lowFont.stringWidth("Fase "
								+ nFase + ": " + nomFase.toString())) / 2,
						Utils.inicioPantallaY, Graphics.TOP | Graphics.LEFT);
			g.setFont(f);
		} else if (!Utils.cargandoPartida && !Utils.guardandoPartida) {
			redibujar = false;
			switch (parteJuego) {
			case video://Men principal
				if (actual != video)
					cargar(g, video);
				else
					getVideo(g);
				break;
			case juego://Memoria:
				if (actual != juego)
					cargar(g, juego);
				else
					getJuego(g);
				break;
			case conversacion://Acerca de...
				if (actual != conversacion)
					cargar(g, conversacion);
				else
					getConversacion(g);
				break;
			case mapa:
				if (actual != mapa)
					cargar(g, mapa);
				else
					getMapa(g);
				break;
			case -1:
				cargarFase(g, nFase);
				break;

			}
			//Ahora dibujo el enlace a men que estar siempre activo.
			if(actual!=-1)
				getAccMenu(g);

		}
		flushGraphics();
	}

	/**
	 * Mtodo encargado de cargar el estado del juego al inicio de una fase.
	 * Adems llama a drawScreen para que pinte la imagen de cargando.
	 * 
	 * @param g
	 *            objeto Graphics necesario para llamar a drawScreen y que de
	 *            esta manera se cargue la imagen de cargando
	 * @param fase
	 *            a cargar
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	private void cargarFase(Graphics g, int fase)
			throws XmlPullParserException, IOException {
		//Cargamos el sprite de Cargar Fase, que va a ser lo primero que
		// hagamos
		Utils.cargarImagen(Utils.spCargando);
		sprite = null;
		sprite = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		sprite.setFrameSequence(Utils.getFrames(Utils.spCargando));
		sprite.setPosition(Utils.inicioPantallaX, Utils.inicioPantallaY);
		// el pintar la pantalla de negro solo se har una vez.
		g.setColor(0x00000000);
		g.fillRect(0, 0, Utils.width, Utils.height);
		cargando = true;
		//Servir de contador para saber cuantas veces pedimos que se pinte la
		// imagen de
		//cargando, solo se podr
		int aux = 0;
		//Pintamos la imagen de cargando
		drawScreen(g);
		aux++;
		//		Inicializamos el parser
		parser = new KXmlParser();
		parser.setInput(new InputStreamReader(this.getClass()
				.getResourceAsStream("/Fase" + nFase + ".xml")));
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Logica");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Fase");
		nomFase.delete(0, nomFase.length());
		nomFase.append(parser.getAttributeValue(1));
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "parteJuego");
		String parte = parser.nextText();
		parteJuego = getParte(parte);
		parser.require(XmlPullParser.END_TAG, null, "parteJuego");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "apaActiva");
		//			Cargamos el array de apariencias
		int apaActiva = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "apaActiva");
		Utils.actualizarApariencia(apaActiva);
		Utils.cargarImagen(Utils.spOscarFren);
		oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		oscar.setFrameSequence(Utils.getFrames(Utils.spOscarFren));
		while (parser.nextTag() != XmlPullParser.END_TAG) {
			if (parser.getName().equals("Videos")) {
				parser.require(XmlPullParser.START_TAG, null, "Videos");
				while (parser.nextTag() != XmlPullParser.END_TAG) {
					parser.require(XmlPullParser.START_TAG, null, "Video");
					cargarVideosFase();
					System.gc();
					if (aux < 3) {
						drawScreen(g);
						aux++;
					}
				}
				parser.require(XmlPullParser.END_TAG, null, "Videos");
			} else if (parser.getName().equals("Conversaciones")) {
				parser.require(XmlPullParser.START_TAG, null, "Conversaciones");
				while (parser.nextTag() != XmlPullParser.END_TAG) {
					parser.require(XmlPullParser.START_TAG, null,
							"Conversacion");
					cargarConversacionFase();
					System.gc();
					if (aux < 4) {
						drawScreen(g);
						aux++;
					}
				}
				parser.require(XmlPullParser.END_TAG, null, "Conversaciones");
			} else if (parser.getName().equals("Mapas")) {
				parser.require(XmlPullParser.START_TAG, null, "Mapas");
				while (parser.nextTag() != XmlPullParser.END_TAG) {
					parser.require(XmlPullParser.START_TAG, null, "Mapa");
					cargarMapaFase();
					System.gc();
				}
				parser.require(XmlPullParser.END_TAG, null, "Mapas");
				drawScreen(g);
			}
		}
		parser.require(XmlPullParser.END_TAG, null, "Fase");

		cargando = false;
		//		Para que compruebe colisiones.
		teclaPulsada = true;
		if (!Utils.cargandoPartida) {
			parser = null;
			System.gc();
			escenActual = ((Mapa) mapas.search(mapaActual).data)
					.getEscenActual();
		}
		redibujar = true;
	}

	/**
	 * Carga los videos de una determinada Fase.
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private void cargarVideosFase() throws XmlPullParserException, IOException {

		//Vector auxiliar
		Vector textos = new Vector();

		int id = Integer.parseInt(parser.getAttributeValue(0));
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "sigParte");
		int sigParte = getParte(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "sigParte");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "vinyetas");
		StringBuffer vinyeta = new StringBuffer();
		vinyeta = getDatosImagen();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "vinyetas");
		if (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "texto");
			textos = cargarTextoVideo();
			parser.nextTag();
			parser.require(XmlPullParser.END_TAG, null, "Video");
		} else
			parser.require(XmlPullParser.END_TAG, null, "Video");

		videosFase.insert(new Video(id, vinyeta.toString(), textos, sigParte));
	}

	/**
	 * Carga el texto de un video
	 * 
	 * @return vector con los textos
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private Vector cargarTextoVideo() throws XmlPullParserException,
			IOException {

		Vector aux = new Vector();

		while (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "valor");
			String valor = parser.nextText();
			aux.addElement(valor);
			parser.require(XmlPullParser.END_TAG, null, "valor");
		}
		parser.require(XmlPullParser.END_TAG, null, "texto");

		return aux;
	}

	/**
	 * Carga las conversaciones de una determinada Fase.
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private void cargarConversacionFase() throws XmlPullParserException,
			IOException {
		//		Vector auxiliar
		Vector opciones = new Vector();

		int id = Integer.parseInt(parser.getAttributeValue(0));
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "color");
		String color = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "color");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Opciones");
		while (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "Opcion");
			opciones.addElement(cargarOpcionObjeto());
		}
		parser.require(XmlPullParser.END_TAG, null, "Opciones");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "Conversacion");

		converFase.insert(new Conversacion(id, getColorHex(color), opciones));
	}

	/**
	 * Carga las estancias de una fase
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private void cargarMapaFase() throws XmlPullParserException, IOException {
		AVLTree estancias = new AVLTree();
		int id = Integer.parseInt(parser.getAttributeValue(0));
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "representa");
		StringBuffer representa = new StringBuffer();
		representa = getDatosImagen();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "representa");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "actual");
		int actual = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "actual");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Estancias");
		while (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "Estancia");
			estancias.insert(cargarEstanciaMapa());
		}
		parser.require(XmlPullParser.END_TAG, null, "Estancias");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "Mapa");
		mapas.insert(new Mapa(id, representa.toString(), estancias, actual));
	}

	/**
	 * Carga las estancias de una fase
	 * 
	 * @return Estancia que forma parte de un Mapa
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private Estancia cargarEstanciaMapa() throws XmlPullParserException,
			IOException {
		AVLTree escenarios = new AVLTree();

		Vector acciones = new Vector();
		int id = Integer.parseInt(parser.getAttributeValue(0));
		String nombre = parser.getAttributeValue(1);
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "representa");
		StringBuffer representa = new StringBuffer();
		representa = getDatosImagen();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "representa");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "actual");
		int actual = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "actual");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "posX");
		int posX = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "posX");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "posY");
		int posY = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "posY");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "visible");
		boolean visible = false;
		if (parser.nextText().equals("SI"))
			visible = true;
		parser.require(XmlPullParser.END_TAG, null, "visible");

		while (parser.nextTag() == XmlPullParser.START_TAG) {
			if (parser.getName().equals("Opciones")) {
				parser.require(XmlPullParser.START_TAG, null, "Opciones");
				while (parser.nextTag() != XmlPullParser.END_TAG) {
					parser.require(XmlPullParser.START_TAG, null, "Opcion");
					acciones.addElement(cargarOpcionObjeto());
				}
			} else if (parser.getName().equals("Escenarios")) {
				parser.require(XmlPullParser.START_TAG, null, "Escenarios");
				while (parser.nextTag() != XmlPullParser.END_TAG) {
					parser.require(XmlPullParser.START_TAG, null, "Escenario");
					escenarios.insert(cargarEscenarioEstancia());
				}
			}
		}

		parser.require(XmlPullParser.END_TAG, null, "Estancia");
		return new Estancia(id, representa.toString(), nombre, actual, posX,
				posY, acciones, escenarios, visible);

	}

	/**
	 * Carga un escenario de una determinada Estancia
	 * 
	 * @return Escenario que forma parte de una Estancia
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private Escenario cargarEscenarioEstancia() throws XmlPullParserException,
			IOException {
		AVLTree items = new AVLTree();

		int id = Integer.parseInt(parser.getAttributeValue(0));
		String nombre = parser.getAttributeValue(1);
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "representa");
		StringBuffer representa = new StringBuffer();
		representa = getDatosImagen();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "representa");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "posX");
		int posX = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "posX");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "posY");
		int posY = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "posY");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "delante");
		boolean delante = false;
		if (parser.nextText().equals("SI"))
			delante = true;
		parser.require(XmlPullParser.END_TAG, null, "delante");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Items");
		while (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "Item");
			items.insert(cargarItemEscenario());
			System.gc();
		}
		parser.require(XmlPullParser.END_TAG, null, "Items");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "Escenario");
		return new Escenario(id, nombre, representa.toString(), posX, posY,
				items, delante);
	}

	/**
	 * Carga un escenario de una determinada Estancia
	 * 
	 * @return Escenario que forma parte de una Estancia
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private Item cargarItemEscenario() throws XmlPullParserException,
			IOException {
		Vector acciones = new Vector();
		AVLTree combinaciones = new AVLTree();

		int id = Integer.parseInt(parser.getAttributeValue(0));
		String nombre = parser.getAttributeValue(1);
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "imagen");
		StringBuffer imagen = new StringBuffer();
		imagen = getDatosImagen();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "imagen");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "posX");
		int posX = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "posX");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "posY");
		int posY = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "posY");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "visible");
		boolean visible = false;
		if (parser.nextText().equals("SI"))
			visible = true;
		parser.require(XmlPullParser.END_TAG, null, "visible");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "animado");
		boolean animado = false;
		if (parser.nextText().equals("SI"))
			animado = true;
		parser.require(XmlPullParser.END_TAG, null, "animado");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "tmpAnima");
		int tmpAnima = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "tmpAnima");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "hayEspecial");
		StringBuffer imagenEspecial = new StringBuffer();
		boolean hayImEspecial = false;
		imagenEspecial.append(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "hayEspecial");
		if (imagenEspecial.toString().equals("SI")) {
			imagenEspecial.delete(0, imagenEspecial.length());
			hayImEspecial = true;
			parser.nextTag();
			parser.require(XmlPullParser.START_TAG, null, "imagenEspecial");
			imagenEspecial = getDatosImagen();
			parser.nextTag();
			parser.require(XmlPullParser.END_TAG, null, "imagenEspecial");
		}
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "posicion");
		int posicion = Integer.parseInt(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "posicion");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "combinable");
		boolean combinable = false;
		if (parser.nextText().equals("SI"))
			combinable = true;
		parser.require(XmlPullParser.END_TAG, null, "combinable");
		if (combinable) {
			parser.nextTag();
			parser.require(XmlPullParser.START_TAG, null, "combinaciones");
			while (parser.nextTag() == XmlPullParser.START_TAG) {
				parser.require(XmlPullParser.START_TAG, null, "combinacion");
				combinaciones.insert(cargarCombinacionItem());
			}
			parser.require(XmlPullParser.END_TAG, null, "combinaciones");
		}
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Opciones");
		while (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "Opcion");
			acciones.addElement(cargarOpcionObjeto());
		}
		parser.require(XmlPullParser.END_TAG, null, "Opciones");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "Item");
		Utils.cargarImagen(imagen.toString());
		if (hayImEspecial) {
			return new Item(nombre, imagen.toString(), posX, posY, visible,
					animado, tmpAnima, imagenEspecial.toString(), acciones,
					posicion, id, combinable, combinaciones);
		} else {
			return new Item(nombre, imagen.toString(), posX, posY, visible,
					animado, tmpAnima, null, acciones, posicion, id,
					combinable, combinaciones);
		}
	}

	/**
	 * Carga una combinacin de un determinado Item con otro Item
	 * 
	 * @return Combinacion de un determinado Item con otro Item
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private Combinacion cargarCombinacionItem() throws XmlPullParserException,
			IOException {
		Vector aux = new Vector();

		int id = Integer.parseInt(parser.getAttributeValue(0));
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "activa");
		boolean activa = false;
		if (parser.nextText().equals("SI"))
			activa = true;
		parser.require(XmlPullParser.END_TAG, null, "activa");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Consecuencias");
		while (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "consecuencia");
			aux.addElement(cargarConsecuenciaAccion());
		}
		parser.require(XmlPullParser.END_TAG, null, "Consecuencias");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "combinacion");
		//		Runtime rt=Runtime.getRuntime() ;
		//		midlet.mostrarAlerta("Combinacion libre= "+id+" ::
		// "+(rt.freeMemory()),AlertType.ERROR);
		return new Combinacion(id, activa, aux);
	}

	/**
	 * Carga la opcin de un determinado Objeto
	 * 
	 * @return opcin que forma parte de un Objeto
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private Opcion cargarOpcionObjeto() throws XmlPullParserException,
			IOException {
		Vector aux = new Vector();

		String id = parser.getAttributeValue(0);
		String texto = parser.getAttributeValue(1);
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "activa");
		boolean activa = false;
		if (parser.nextText().equals("SI"))
			activa = true;
		parser.require(XmlPullParser.END_TAG, null, "activa");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Consecuencias");
		while (parser.nextTag() == XmlPullParser.START_TAG) {
			parser.require(XmlPullParser.START_TAG, null, "consecuencia");
			aux.addElement(cargarConsecuenciaAccion());
		}
		parser.require(XmlPullParser.END_TAG, null, "Consecuencias");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "Opcion");
		//		Runtime rt=Runtime.getRuntime() ;
		//		midlet.mostrarAlerta("Opcion libre= "+id+" ::
		// "+(rt.freeMemory()),AlertType.ERROR);
		return new Opcion(activa, texto, id, aux);

	}

	/**
	 * Cargar las consecuencias de una accin
	 * 
	 * @return Vector con un conjunto de resultados que formarn una
	 *         consecuencia
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private Vector cargarConsecuenciaAccion() throws XmlPullParserException,
			IOException {

		Vector resultado = new Vector();
		StringBuffer id = new StringBuffer();
		StringBuffer info = new StringBuffer();
		while (parser.nextTag() == XmlPullParser.START_TAG) {
			id.delete(0, id.length());
			info.delete(0, info.length());
			parser.require(XmlPullParser.START_TAG, null, "Resultado");
			id.append(parser.getAttributeValue(0));
			info.append(parser.nextText());
			parser.require(XmlPullParser.END_TAG, null, "Resultado");
			resultado.addElement(new Resultado(Utils.getIdResultado(id
					.toString()), info.toString()));
		}
		parser.require(XmlPullParser.END_TAG, null, "consecuencia");
		//		Runtime rt=Runtime.getRuntime() ;
		//		midlet.mostrarAlerta("Escenario libre= "+id+" ::
		// "+(rt.freeMemory()),AlertType.ERROR);
		return resultado;
	}

	/**
	 * Recupera, a partir de un archivo XML, los datos descriptivos de una
	 * imagen
	 * 
	 * @throws IOException
	 *             Excepcin de entrada/salida
	 * @throws XmlPullParser
	 *             Exception Excepcin en el XML
	 */
	public StringBuffer getDatosImagen() throws XmlPullParserException,
			IOException {

		StringBuffer imagen = new StringBuffer();
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "id");
		imagen.append(parser.nextText() + ";");
		parser.require(XmlPullParser.END_TAG, null, "id");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "frameIni");
		imagen.append(parser.nextText() + ";");
		parser.require(XmlPullParser.END_TAG, null, "frameIni");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "numFrames");
		imagen.append(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "numFrames");
		return imagen;
	}

	/**
	 * Decide que y cuando cargar en cada momento
	 * 
	 * @param g
	 *            Objeto Graphics
	 * @param pantalla
	 *            pantalla que se desea cargar
	 */
	private void cargar(Graphics g, int pantalla) {
		switch (pantalla) {
		case video:
			if (actual == conversacion) {
				descargarConversacion();
			}
			if (actual == juego) {
				descargarEscenario();
			}
			actual = video;
			Video aux = (Video) videosFase.search(videoSiguiente).data;
			//cargamos las imgenes del video
			Utils.cargarImagen(aux.vinyeta);
			sprite.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			sprite.setFrameSequence(Utils.getFrames(aux.vinyeta));
			sprite.setPosition(Utils.inicioPantallaX, Utils.inicioPantallaY);
			//Espacio entre cada una de las lneas de la vieta
			Utils.spacing = (Utils.lowFont.getHeight() * 85) / 100;
			//Inicializamos lnea
			Utils.linea = 0;

			g.setFont(Utils.lowFont);
			g.setColor(Utils.highColor);
			//Calcula la altura y anchura a la que empieza el texto de la
			// vieta
			Utils.startHeight = (Utils.height / 2) - (Utils.getTamGrupoY() / 2);
			Utils.startWidth = (Utils.width / 2) - (Utils.getTamGrupoX() / 2)
					+ Utils.lowFont.getSize();
			//Calculamos el numero de lneas y de letras que caben
			//Cambiamos el numero de lneas que caben en pantalla para los
			// videos
			//el 33% de 128 es 42 (33% de 200 es 66) mas o menos, y ese es
			//el espacio que dejaremos para el texto de la vieta.
			int espacioTexto = (Utils.getTamGrupoY() * 33) / 100;
			Utils.numLineas = espacioTexto / Utils.lowFont.getHeight() - 1;
			Utils.spaceMen = Utils.getTamGrupoX();
			if (colisionado) {
				colisionado = false;
				letra = -6;
				puntero.nextFrame();
			}
			break;
		case juego:
			if (actual == mapa)
				descargarMapa();
			else if (actual == conversacion) {
				descargarConversacion();
			}
			actual = juego;
			//Inicializamos lnea
			Utils.linea = 0;
			if (Utils.grupo == 1)//Si el movil es de grupo grande
				g.setFont(Utils.highFont);
			//Calcula la altura y el ancho a la que se mostrar el nombre de
			// los items
			Utils.startHeight = (Utils.height / 2) - (Utils.getTamGrupoY() / 2)
					+ (Utils.getTamGrupoY() - Utils.getSpaceTextItem());
			Utils.startWidth = (Utils.width / 2) - (Utils.getTamGrupoX() / 2)
					+ Utils.lowFont.getSize();
			sprite = null;
			//cargamos sprite con una imagen del tamao de inventario
			Utils.cargarImagen(Utils.imTmpInv);
			sprite = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			puntero.setPosition(Utils.inicioPantallaX
					+ (Utils.getTamGrupoX() / 2), Utils.inicioPantallaY
					+ (Utils.getTamGrupoY() / 2));
			seleccionado = null;
			if (colisionado) {
				colisionado = false;
				letra = -6;
				puntero.nextFrame();
			}
			cargarEscenario();
			break;
		case conversacion:
			if (actual == juego) {
				descargarEscenario();
			}
			actual = conversacion;
			tmpAcumulado = 0;
			longIdOp = 1;
			participante = (Item) seleccionado;
			seleccionado = (Opcionable) converFase.search(converSelec).data;
			Utils.spacing = (Utils.lowFont.getHeight() * 85) / 100;
			//Cargamos y colocamos el Sprite de Oscar.
			Utils.cargarImagen(Utils.spOscarCara);
			oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			oscar.setFrameSequence(Utils.getFrames(Utils.spOscarCara));
			oscar.setPosition(Utils.inicioPantallaX, Utils.inicioPantallaY);
			//Cargamos y colocamos el sprite del otro que habla
			sprite = null;
			Utils.cargarImagen(participante.imagenEspecial);
			sprite = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			sprite.setFrameSequence(Utils
					.getFrames(participante.imagenEspecial));
			sprite.setPosition(Utils.inicioPantallaX + oscar.getWidth(),
					Utils.inicioPantallaY);
			g.setFont(Utils.lowFont);
			//Calcula la altura y anchura a la que empieza el texto de la
			// conversacin
			Utils.startHeight = Utils.inicioPantallaY + oscar.getHeight();
			Utils.startWidth = Utils.inicioPantallaX
					+ ((oscar.getWidth() * 32) / 100);
			Utils.numLineas = (Utils.getTamGrupoY() - oscar.getHeight())
					/ Utils.lowFont.getHeight() - 1;
			Utils.spaceMen = Utils.getTamGrupoX()
					- ((oscar.getWidth() * 60) / 100);
			opcionSele = 0;
			mostrarOpciones = true;
			if (colisionado) {
				colisionado = false;
				letra = -6;
				puntero.nextFrame();
			}
			break;
		case mapa:
			if (actual == conversacion) {
				descargarConversacion();
			}
			if (actual == juego) {
				descargarEscenario();
			}
			actual = mapa;
			if (Utils.grupo == 1)//Si el movil es de grupo grande
				g.setFont(Utils.highFont);
			//Calcula la altura y el ancho a la que se mostrar el nombre de
			// los items
			Utils.startHeight = (Utils.height / 2) - (Utils.getTamGrupoY() / 2)
					+ (Utils.getTamGrupoY() - Utils.getSpaceTextItem());
			Utils.startWidth = (Utils.width / 2) - (Utils.getTamGrupoX() / 2)
					+ Utils.lowFont.getSize();
			puntero.setPosition(Utils.inicioPantallaX
					+ (Utils.getTamGrupoX() / 2), Utils.inicioPantallaY
					+ (Utils.getTamGrupoY() / 2));
			seleccionado = null;
			if (colisionado) {
				colisionado = false;
				letra = -6;
				puntero.nextFrame();
			}
			cargarMapa();
			break;
		}
		redibujar = true;
	}

	/**
	 * Dibuja la pantalla del video
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getVideo(Graphics g) {
		Video video = (Video) videosFase.search(videoSiguiente).data;
		//pintamos el sprite
		sprite.paint(g);
		//Imprimimos el mensaje
		mostrarMensaje((String) video.texto[sprite.getFrame()], "SIGUIENTE", g);
	}

	/**
	 * Dibuja la pantalla de juego
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getJuego(Graphics g) {

		//		Color blanco para los nombres de los Items
		g.setColor(Utils.lowColor);
		LM.paint(g, 0, 0);
		//el inventario no se muestra hasta que no estn todos los resultados
		//de un determinado Item, ejecutados, porque si no se puede dar el caso
		//de que hagan reAddInventInt y entonces cambia el seleccionado
		//A no ser que el inventario ya estuviese siendo mostrado
		if (mostrarInventario) {
			getInventario(g);
			if (itemsInventario.size() != 0) {
				g.setColor(0x00DDDDDD);
				getLineatexto(g, ((Item) seleccionado).nombre);
			}
			if (mostrarOpciones)
				getOpciones(g);
		}

		if (mostrarMensaje)
			getMensaje(g);

		//si lo seleccionado es un Item
		if (colisionado && seleccionado != null) {
			g.setColor(0x00DDDDDD);
			getLineatexto(g, ((Item) seleccionado).nombre);
			if (mostrarOpciones) {
				getOpciones(g);
			}
			g.setColor(Utils.lowColor);
		}
		//si la colisin no ha sido con un Item, pues habr sido con Oscar
		else if (colisionado && seleccionado == null)
			getLineatexto(g, "Oscar Romero");
		else if (!mostrarInventario && !mostrarOpciones)//si no sealas nada
			getLineatexto(g, "Ir a...");
	}

	/**
	 * Dibuja el inventario
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getInventario(Graphics g) {
		//		Dibujamos el rectngulo marrn
		g.setColor(0x996633);
		int tempX = Utils.inicioPantallaX;
		int tempY = Utils.inicioPantallaY + Utils.getTamGrupoY()
				- Utils.getSpaceTextItem() - altoInventario;
		g.fillRect(tempX, tempY, anchoInventario, altoInventario);
		//Dibujamos el cuadrado rojo (tiene mismo ancho y alto claro)
		g.setColor(0x00DD0000);
		int tempX2 = Utils.inicioPantallaX + (anchoInventario / 2)
				- (altoInventario / 2);
		g.fillRect(tempX2, tempY, altoInventario, altoInventario);
		//Ahora las flechas
		g.setColor(Utils.highColor);
		g.fillTriangle(tempX2 - (anchoInventario / 20), tempY
				+ (altoInventario / 14), Utils.inicioPantallaX, tempY
				+ (altoInventario / 2), tempX2 - (anchoInventario / 20), tempY
				+ (altoInventario) - (altoInventario / 14));

		g.fillTriangle(tempX2 + altoInventario + (anchoInventario / 20), tempY
				+ (altoInventario / 14), Utils.inicioPantallaX
				+ anchoInventario, tempY + (altoInventario / 2), tempX2
				+ altoInventario + (anchoInventario / 20), tempY
				+ (altoInventario) - (altoInventario / 14));
		//Ahora mostramos el inventario
		if (itemsInventario.size() != 0) {
			//Si por casualidad acabamos de sacar del inventario, el objeto que
			//estaba seleccionado por invSelec, lo corregimos aqu.
			if (invSelec >= itemsInventario.size())
				invSelec = 0;
			seleccionado = (Item) itemsInventario.elementAt(invSelec);
			//			cargamos la imagen del inventario para ese objeto
			Utils.cargarImagen(((Item) seleccionado).imagenEspecial);
			sprite.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			tempY = tempY + (altoInventario / 2) - (sprite.getHeight() / 2);
			tempX = tempX2 + (altoInventario / 2) - (sprite.getWidth() / 2);
			sprite.setPosition(tempX, tempY);
			sprite.paint(g);
		} else {
			tempY = tempY + (altoInventario / 2) - (sprite.getHeight() / 2);
			tempX = tempX2 + (altoInventario / 2) - (sprite.getWidth() / 2);
			//dibujamos cuadrado negro con la palabra Vaco en el centro.
			g.setColor(Utils.highColor);
			g.fillRect(tempX, tempY, sprite.getWidth(), sprite.getHeight());
			g.setColor(Utils.lowColor);
			g.drawString("VACIO", tempX + sprite.getWidth() / 2
					- (Utils.highFont.charWidth('a') * ("Vacio".length() / 2)),
					tempY + (sprite.getHeight() / 2)
							- Utils.highFont.getHeight(), Graphics.TOP
							| Graphics.LEFT);
		}
	}

	/**
	 * Dibuja la pantalla de la conversacin
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getConversacion(Graphics g) {
		//indica que tecla se ha tocado, si el botn izquiera o el derecha.
		boolean izquierda = false;
		//indica si se ha comprobado si el botn pulsado fue el izquierda o el
		// derecha.
		boolean comprobado = false;
		Conversacion con = ((Conversacion) seleccionado);
		//Ponemos la pantalla en blaco
		g.setColor(0xFFFFFFFF);
		g.fillRect(Utils.inicioPantallaX, Utils.inicioPantallaY, Utils
				.getTamGrupoX(), Utils.getTamGrupoY());
		//Dibujamos el cuadrado, en el color correspondiente,
		//del otro que habla
		g.setColor(con.color);
		g.fillRect(Utils.inicioPantallaX + oscar.getWidth(),
				Utils.inicioPantallaY, sprite.getWidth(), sprite.getHeight());
		g.setColor(Utils.highColor);
		if (mostrarOpciones) {
			//			Dibujamos los rectangulos marrones
			g.setColor(0x996633);
			g.fillRect(Utils.inicioPantallaX, Utils.inicioPantallaY
					+ oscar.getHeight(), (oscar.getWidth() * 30) / 100, Utils
					.getTamGrupoY()
					- oscar.getHeight());
			g.fillRect(Utils.inicioPantallaX + Utils.getTamGrupoX()
					- ((oscar.getWidth() * 30) / 100), Utils.inicioPantallaY
					+ oscar.getHeight(), (oscar.getWidth() * 30) / 100, Utils
					.getTamGrupoY()
					- oscar.getHeight());
			//Dibujamos las flechas
			g.setColor(Utils.highColor);
			g.fillTriangle(Utils.inicioPantallaX
					+ ((oscar.getWidth() * 25) / 100), Utils.inicioPantallaY
					+ oscar.getHeight() + 2, Utils.inicioPantallaX,
					Utils.inicioPantallaY + oscar.getHeight()
							+ ((Utils.getTamGrupoY() - oscar.getHeight()) / 2),
					Utils.inicioPantallaX + ((oscar.getWidth() * 25) / 100),
					Utils.inicioPantallaY + Utils.getTamGrupoY() - 2);

			int tempX = Utils.inicioPantallaX + Utils.getTamGrupoX();
			g.fillTriangle(tempX - ((oscar.getWidth() * 25) / 100),
					Utils.inicioPantallaY + oscar.getHeight() + 2, tempX,
					Utils.inicioPantallaY + oscar.getHeight()
							+ ((Utils.getTamGrupoY() - oscar.getHeight()) / 2),
					tempX - ((oscar.getWidth() * 25) / 100),
					Utils.inicioPantallaY + Utils.getTamGrupoY() - 2);
			//Ahora hayamos las opciones que pueden ser mostradas
			//salir se pondr a true cuando se cumplan las condiciones de una
			// opcin
			boolean salir = false;
			while (!salir) {
				if (con.opciones[opcionSele] != null) {
					if (con.opciones[opcionSele].id.length() == longIdOp) {
						if (longIdOp == 1)
							salir = true;
						else {
							if ((con.opciones[opcionSele].id.substring(0,
									longIdOp - 1).equals(iniIdOp.toString())))
								salir = true;
							else {
								if (opcionSele < con.opciones.length - 1)
									opcionSele++;
								else
									opcionSele = 0;
							}
						}
					} else {
						if (opcionSele < con.opciones.length - 1)
							opcionSele++;
						else
							opcionSele = 0;
					}
				} else {
					if (opcionSele < con.opciones.length - 1)
						opcionSele++;
					else
						opcionSele = 0;
				}
				if (con.opciones[opcionSele] != null) {
					if (!seleccionado.getAccionActiva(opcionSele)
							&& salir == true) {
						if (opcionSele < con.opciones.length - 1)
							opcionSele++;
						else
							opcionSele = 0;
						salir = false;
					}
				}
			}
			//			Ahora imprimimos la opcin seleccionada
			Utils.mensaje.delete(0, Utils.mensaje.length());
			Utils.mensaje.append(con.opciones[opcionSele].texto);
		}
		//Dibujamos una lnea negra entre las 2 caras
		g.drawLine(Utils.inicioPantallaX + oscar.getWidth(),
				Utils.inicioPantallaY,
				Utils.inicioPantallaX + oscar.getWidth(), Utils.inicioPantallaY
						+ oscar.getHeight());
		//Dibujamos una lnea negra entre el espacio de las caras y el del
		//texto
		g.drawLine(Utils.inicioPantallaX, Utils.inicioPantallaY
				+ oscar.getHeight(), Utils.inicioPantallaX
				+ Utils.getTamGrupoX(), Utils.inicioPantallaY
				+ oscar.getHeight());
		//introducimos sus caras
		oscar.paint(g);
		sprite.paint(g);
		if (habParti)
			g.setColor(con.color);
		//mostramos el mensaje
		if (mostrarOpciones)
			mostrarMensaje(Utils.mensaje.toString(), "DECIR", g);
		else
			mostrarMensaje(Utils.mensaje.toString(), "CONTINUAR", g);
	}

	/**
	 * Dibuja la pantalla del mapa
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getMapa(Graphics g) {
		//		Color blanco para los nombres de las Estancia
		g.setColor(Utils.lowColor);
		LM.paint(g, 0, 0);

		if (mostrarMensaje)
			getMensaje(g);

		//si lo seleccionado es una Estancia
		if (colisionado && seleccionado != null) {
			g.setColor(0x00DDDDDD);
			getLineatexto(g, ((Estancia) seleccionado).nombre);
			if (mostrarOpciones) {
				getOpciones(g);
			}
			g.setColor(Utils.lowColor);
		}

	}

	/**
	 * Dibuja el acceso directo al men.
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getAccMenu(Graphics g) {
		int fontStyle = g.getFont().getStyle();
		g.setFont(Utils.highFont);
		/*
		 * if(Utils.grupo==0) g.setFont(Utils.lowFont);
		 */
		g.setColor(getColorHex("Crema"));
		g.fillRect(posXMenu, posYMenu, anchoMenu, altoMenu);
		g.setColor(getColorHex("Rojo"));
		g.drawRect(posXMenu, posYMenu, anchoMenu, altoMenu);
		g.setColor(Utils.highColor);
		g.drawString("Menu", posXMenu + 2, posYMenu - 2, Graphics.TOP
				| Graphics.LEFT);
		//en caso de que sea necesario ponemos la antigua fuente
		if (fontStyle == Font.STYLE_PLAIN)
			g.setFont(Utils.lowFont);
	}

	/**
	 * Realiza la carga del escenario de juego.
	 */
	private void cargarEscenario() {
		if (!escenActual.nombre.equals(ultimoEscenario.toString())) {
			//para que no entre en el if que hay mas abajo y asi lo ponga en la
			//posicin inicial del mapa
			pasoActual = 0;
			cargarAreaCaminable();
			ultimoEscenario.delete(0, ultimoEscenario.length());
			ultimoEscenario.append(escenActual.nombre);
		}
		escenDetras = null;
		escenDetras = new Sprite(escenActual.representa);
		escenDetras.setFrame(escenActual.imDetras);
		LM.append(puntero);
		Item temp2;
		Node aux = escenActual.items.root;
		escenActual.items.inorden(aux);
		aux = escenActual.items.next();
		while (aux != null) {
			temp2 = (Item) aux.data;
			if (temp2.posicion == 2)
				//Primero metemos los Items de delante
				LM.append(temp2);
			aux = escenActual.items.next();
		}
		//Luego el escenario de delante
		if (escenActual.delante) //si tiene alguna parte del escenario que va
			// delante...
			LM.append(escenActual.representa);
		//introducimos a Oscar
		//Carga la posicin de oscar en el escenario
		oscar.setPosition(Utils.inicioPantallaX + escenActual.getPosInicialX(),
				Utils.inicioPantallaY + escenActual.getPosInicialY());
		//pero si nos encontramos en el mismo escenario que antes
		//y oscar se haba movido, cargamos su ultima posicin
		if (pasoActual > 0) {
			int x = getPixelX(buscaRuta.getX(buscaRuta.path[pasoActual]));
			int y = getPixelY(buscaRuta.getY(buscaRuta.path[pasoActual]));
			oscar.setPosition(x, y);
		}
		LM.append(oscar);
		aux = escenActual.items.root;
		escenActual.items.inorden(aux);
		aux = escenActual.items.next();
		while (aux != null) {
			temp2 = (Item) aux.data;
			if (temp2.posicion == 1)
				//				Luego aadimos los Items de detrs de Oscar
				LM.append(temp2);
			aux = escenActual.items.next();
		}
		//		Luego el escenario de detrs del prota
		LM.append(escenDetras);
		//el siguiente sprite ser el rea caminable
		//LM.append(escenCaminable);//El rea caminable ya no lo metemos,
		// porque
		//esta precalculado.
		aux = escenActual.items.root;
		escenActual.items.inorden(aux);
		aux = escenActual.items.next();
		while (aux != null) {
			temp2 = (Item) aux.data;
			if (temp2.posicion == 0)
				//				Por ltimo los items de detrs del escenario.
				LM.append(temp2);
			aux = escenActual.items.next();
		}
		//Para que compruebe colisiones.
		teclaPulsada = true;
		letra = -6;
	}

	/**
	 * Carga el rea caminable del escenario actual.
	 */
	private void cargarAreaCaminable() {
		//lo primero en cargar es el sprite del puntero que comprueba el rea
		// caminable
		//el cual tendr un tamao de 2x2
		//guardamos la posicin actual del puntero
		int tempX = puntero.getX();
		int tempY = puntero.getY();
		int frame = puntero.getFrame();

		Utils.cargarImagen(Utils.imPunCami);
		puntero.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		LM.append(puntero);

		escenCaminable = null;
		escenCaminable = new Sprite(escenActual.representa);
		escenCaminable.setFrame(escenActual.imCaminable);

		//el siguiente sprite ser el rea caminable
		LM.append(escenCaminable);
		//Calculamos para ahorrrnoslo en el bucle for
		int temPosIniX = Utils.inicioPantallaX + (anchoCelda / 2);
		int temPosIniY = Utils.inicioPantallaY + (altoCelda / 2);

		int j = 0;
		boolean caminable = false;
		int maxi = buscaRuta.walkability.length;
		int maxj = buscaRuta.mapWidth;

		//walkability hay que modificarlo para indicar cuales de las celdas son
		//caminables y cuales no. Si establecemos true, es caminable , y si no
		// pues no.
		//En este for lo que hace es comprobar una celda y si esa celda es
		// caminable,
		//todas las conectadas directamente a ella, tb lo sern. Y viceversa.
		//Es decir si el ancho y alto de cada celda es de 2x2, entonces aqu
		//simularemos que son de 14x14.Un cuadrado de celdas de 7x7
		//Ejemplo, empezamos por la celda 4,4 --> Si es caminable, pues 0,0 0,1
		// 0,2
		//0,3 0,4 0,5 0,6 1,0 1,1 1,2... 6,3 6,4 6,5 6,6 tb sern caminables, y
		// la siguiente en mirar
		//ser la celda 4,11. Si llegsemos al final de la fila 4, pasaramos a
		// mirar
		//la celda 11,4
		//Puede pasar que las ultimas filas (como mucho 4)queden sin mirar, si
		// son
		//caminables o no, pero no importa, ya que esas filas corresponden al
		// rea
		//donde se pone el nombre de los items, y eso nunca es caminable.

		//Lado del cuadrado imaginario que vamos a formar para de 7x7.
		//Debe ser siempre un numero impar, para que la celda del centro este
		//justo en el centro
		int ladoCuadrado = 7;
		for (int i = 3 * maxj; i < maxi; i++) {
			for (j = 3; j < maxj; j = j + ladoCuadrado) {
				puntero.setPosition(temPosIniX + j * anchoCelda, temPosIniY
						+ ((i / buscaRuta.mapWidth) * altoCelda));
				caminable = puntero.collidesWith(escenCaminable, true);
				int numRecorre = (ladoCuadrado / 2) + 1;
				for (int k = 0; k < numRecorre; k++) {
					for (int z = 0; z < numRecorre; z++) {
						if (i + j + k * maxj + z < maxi)
							buscaRuta.walkability[i + j + k * maxj + z] = caminable;
						if (i + j + k * maxj - z < maxi)
							buscaRuta.walkability[i + j + k * maxj - z] = caminable;
						if (i + j - k * maxj + z < maxi && k != 0)
							buscaRuta.walkability[i + j - k * maxj + z] = caminable;
						if (i + j - k * maxj - z < maxi && k != 0)
							buscaRuta.walkability[i + j - k * maxj - z] = caminable;
					}
				}
			}
			//Multiplicamos por 3, porque al mirar en cuadrado, iremos mirando
			//la fila 1 , la 4, la 7...
			i = i + (maxj * 7) - 1;
		}
		LM.remove(puntero);
		LM.remove(escenCaminable);
		//restablecemos la posicin e imagen del puntero
		Utils.cargarImagen(Utils.spPuntero);
		puntero.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		puntero.setFrameSequence(Utils.getFrames(Utils.spPuntero));
		puntero.setFrame(frame);
		puntero.setPosition(tempX, tempY);
	}

	/**
	 * Realiza la descarga del escenario de juego.
	 */
	private void descargarEscenario() {
		//lo primero en cargar es el sprite del puntero
		LM.remove(puntero);
		Item temp2;

		Node aux = escenActual.items.root;
		escenActual.items.inorden(aux);
		aux = escenActual.items.next();
		while (aux != null) {
			temp2 = (Item) aux.data;
			if (temp2.posicion == 2)
				//Primero metemos los Items de delante
				LM.remove(temp2);
			aux = escenActual.items.next();
		}
		//Luego el escenario de delante
		if (escenActual.delante) //si tiene alguna parte del escenario que va
			// delante...
			LM.remove(escenActual.representa);

		LM.remove(oscar);

		aux = escenActual.items.root;
		escenActual.items.inorden(aux);
		aux = escenActual.items.next();
		while (aux != null) {
			temp2 = (Item) aux.data;
			if (temp2.posicion == 1)
				//				Luego borramos los Items de detrs de Oscar
				LM.remove(temp2);
			aux = escenActual.items.next();
		}
		//		Luego el escenario de detrs del prota
		LM.remove(escenDetras);

		aux = escenActual.items.root;
		escenActual.items.inorden(aux);
		aux = escenActual.items.next();
		while (aux != null) {
			temp2 = (Item) aux.data;
			if (temp2.posicion == 0)
				//Por ltimo los items de detrs del escenario.
				LM.remove(temp2);
			aux = escenActual.items.next();
		}
	}

	/**
	 * Realiza la carga del mapa de juego.
	 */
	private void cargarMapa() {
		//Se pone a 0, si al salir al mapa vuelves a entrar al mismo escenario
		//que antes, pues te pondr en la posicin inicial del escenario
		//y no en la posicin que tenas, en ese escenario, antes de salir al
		// mapa.
		pasoActual = 0;
		Mapa temp = (Mapa) mapas.search(mapaActual).data;
		//lo primero en cargar es el sprite del puntero
		LM.append(puntero);
		Estancia temp2;

		//		Cargamos el Mapa
		LM.append(temp.representa);

		//Cargamos la representacin de los distintas estancias, que
		//irn detrs del mapa de tal manera que al pasar el puntero
		//por encima, estas colisionen con el
		Node aux = ((Mapa) mapas.search(mapaActual).data).estancias.root;
		((Mapa) mapas.search(mapaActual).data).estancias.inorden(aux);
		aux = ((Mapa) mapas.search(mapaActual).data).estancias.next();
		while (aux != null) {
			temp2 = (Estancia) aux.data;
			LM.append(temp2.representa);
			aux = ((Mapa) mapas.search(mapaActual).data).estancias.next();
		}

		//		Para que compruebe colisiones.
		teclaPulsada = true;
	}

	/**
	 * Realiza la descarga del mapa de juego.
	 */
	private void descargarMapa() {
		Mapa temp = (Mapa) mapas.search(mapaActual).data;
		//lo primero en cargar es el sprite del puntero
		LM.remove(puntero);
		Estancia temp2;

		//		Descargamos el Mapa
		LM.remove(temp.representa);

		//Descargamos la representacin de los distintas estancias, que
		//irn detrs del mapa de tal manera que al pasar el puntero
		//por encima, estas colisionen con el
		Node aux = ((Mapa) mapas.search(mapaActual).data).estancias.root;
		((Mapa) mapas.search(mapaActual).data).estancias.inorden(aux);
		aux = ((Mapa) mapas.search(mapaActual).data).estancias.next();
		while (aux != null) {
			temp2 = (Estancia) aux.data;
			LM.remove(temp2.representa);
			aux = ((Mapa) mapas.search(mapaActual).data).estancias.next();
		}
	}

	/**
	 * Realiza la descarga de la conversacin
	 */
	private void descargarConversacion() {
		seleccionado = participante;
		participante = null;
		//	Cargamos el Sprite de Oscar.
		Utils.cargarImagen(Utils.spOscarFren);
		oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		oscar.setFrameSequence(Utils.getFrames(Utils.spOscarFren));
		mostrarOpciones = false;
	}

	/**
	 * Comprueba si hay colisin entre el puntero y algn Item del escenario
	 * actual, tambin se incluye a Oscar Solo hace falta que colisione con uno,
	 * para que devuelva verdadero
	 * 
	 * @return true si hay colisin entre el puntero y un Item del escenario
	 *         actual
	 */
	private boolean colisionPunItem() {
		Item aux;
//		en caso de que ahora este colisionado, miramos si sigue en el mismo, siempre
		//que el de ahora sea posicion 2
		if (colisionado  
				&& ((Item)seleccionado).posicion == 2 
				&& puntero.collidesWith((Item)seleccionado, true))
			return true;
		Node auxN = escenActual.items.root;
		escenActual.items.inorden(auxN);
		auxN = escenActual.items.next();
		
		//primero miramos a ver si colisiona primero con los que estn delante
		// de oscar.
		while (auxN != null) {
			aux = (Item) auxN.data;
			if (aux.posicion == 2 && aux.isVisible()) {
				if (puntero.collidesWith(aux, true)) {
					//Si el objeto es distinto al actualmente seleccionado
					//entonces reiniciamos letra
					if (seleccionado != null
							&& ((Item) seleccionado).id != aux.id)
						letra = -6;
					seleccionado = aux;
					System.gc();
					return true;
				}
			}
			auxN = escenActual.items.next();
		}

		//Si queris habilitar la opcin de seleccionar a Oscar con el puntero
		//Descomentar la siguiente sentencia if
		//		En caso de que antes no colisionase,a ver si colisiona con Oscar
		/*
		 * if(puntero.collidesWith(oscar,true)){ seleccionado=null; letra=-6;
		 * System.gc(); return true; }
		 */
		
//		en caso de que ahora este colisionado, miramos si sigue en el mismo, siempre
		//que el de ahora sea posicion 1
		if (colisionado  
				&& ((Item)seleccionado).posicion == 1
				&& puntero.collidesWith((Item)seleccionado, true))
			return true;
	
		//detrs de Oscar.
		auxN = escenActual.items.root;
		escenActual.items.inorden(auxN);
		auxN = escenActual.items.next();
		//	Luego los que estn entre Oscar y el escenario.
		while (auxN != null) {
			aux = (Item) auxN.data;
			if (aux.posicion == 1 && aux.isVisible()) {
				if (puntero.collidesWith(aux, true)) {
					//Si el objeto es distinto al actualmente seleccionado
					//entonces reiniciamos letra
					if (seleccionado != null
							&& ((Item) seleccionado).id != aux.id)
						letra = -6;
					seleccionado = aux;
					System.gc();
					return true;
				}
			}
			auxN = escenActual.items.next();
		}
		
//		en caso de que ahora este colisionado, miramos si sigue en el mismo, siempre
		//que el de ahora sea posicion 0
		if (colisionado  
				&& ((Item)seleccionado).posicion == 0
				&& puntero.collidesWith((Item)seleccionado, true))
			return true;
		
		//Detrs del escenario.
		auxN = escenActual.items.root;
		escenActual.items.inorden(auxN);
		auxN = escenActual.items.next();
		//	Por ltimo los que estn por detrs del escenario
		while (auxN != null) {
			aux = (Item) auxN.data;
			if (aux.posicion == 0 && aux.isVisible()) {
				if (puntero.collidesWith(aux, true)) {
					//Si el objeto es distinto al actualmente seleccionado
					//entonces reiniciamos letra
					if (seleccionado != null
							&& ((Item) seleccionado).id != aux.id)
						letra = -6;
					seleccionado = aux;
					System.gc();
					return true;
				}
			}
			auxN = escenActual.items.next();
		}
		return false;
	}

	/**
	 * Comprueba si hay colisin entre el puntero y alguna Estancia del mapa.
	 * Solo hace falta que colisione con una, para que devuelva verdadero
	 * 
	 * @return true si hay colisin entre el puntero y una Estancia del mapa
	 */
	private boolean colisionPunEstan() {
//		en caso de que ahora este colisionado, miramos si sigue en el mismo
		Sprite aux;
		Node auxN = ((Mapa) mapas.search(mapaActual).data).estancias.root;
		((Mapa) mapas.search(mapaActual).data).estancias.inorden(auxN);
		auxN = ((Mapa) mapas.search(mapaActual).data).estancias.next();
		while (auxN != null) {
			aux = ((Estancia) auxN.data).representa;
			if (puntero.collidesWith(aux, true)) {
				seleccionado = (Estancia) auxN.data;
				return true;
			}
			auxN = ((Mapa) mapas.search(mapaActual).data).estancias.next();
		}
		return false;
	}

	/**
	 * Para aquellos Items que estn establecidos como animados, este mtodo
	 * comprueba si ha llegado su hora de cambiar de frame.
	 */
	private void animarItems() {
		Item aux;
		Node auxN = escenActual.items.root;
		escenActual.items.inorden(auxN);
		auxN = escenActual.items.next();
		while (auxN != null) {
			aux = (Item) auxN.data;
			if (aux.animado && tmpAcumulado % aux.tmpAnima == 0) {
				aux.nextFrame();
				redibujar = true;
			}
			auxN = escenActual.items.next();
		}
	}

	/**
	 * mtodo que salta como respuesta a la pulsacin de una accin, de entre
	 * las disponibles para un objeto.
	 */
	private void opcionesIO() {
		int idOpActivas[] = new int[50];
		int contador = 0;
		int idOp = opcionSele;
		if (actual == juego) {
			for (int i = 0; i < seleccionado.longAcciones(); i++) {
				if (seleccionado.getAccionActiva(i)) {
					idOpActivas[contador] = i;
					contador++;
				}
			}
			idOp = idOpActivas[opcionSele];
		}
		Vector aux;
		if (!seleccionado.getConseActivaAcc(idOp)) {
			aux = (Vector) seleccionado.getConsecuenciaAcc(idOp);
			for (int i = 0; i < aux.size(); i++)
				resultados.addElement(aux.elementAt(i));
		}
	}

	/**
	 * mtodo que salta como respuesta al intento de combinar dos Items
	 */
	private void conseCombinacion() {
		AVLTree arbol;
		Vector aux;
		int idSele = -1;//Combinaciones con id=-1 (la de por defecto)
		if (seleccionado != null
				&& //si no es oscar y
				combinado.combinaciones.search(((Item) seleccionado).id) != null) //es
			// un
			// objeto
			// combinable
			// con
			// este
			if (combinado.getComActiva(((Item) seleccionado).id))//Si la
				// combinacin
				// est activa.
				idSele = ((Item) seleccionado).id;

		aux = (Vector) combinado.getConsecuenciaCom(idSele);
		for (int i = 0; i < aux.size(); i++)
			resultados.addElement(aux.elementAt(i));
	}

	/**
	 * Mtodo encargado de dirigir la ejecucin de cada uno de los resultados de
	 * una acccion
	 * @throws InterruptedException
	 */
	private void ejecutarResultado() throws InterruptedException {
		Resultado aux = (Resultado) resultados.firstElement();
		System.gc();
		switch (aux.id) {
			case Utils.reSalir:
				mostrarOpciones = false;
				opcionSele = 0;
				break;
			case Utils.reCamFase:
				nFase++;
				switch (actual) {
					case juego:
						descargarEscenario();
						break;
					case inventario:
						descargarEscenario();
						break;
					case mapa:
						descargarMapa();
						break;
					case conversacion:
						descargarConversacion();
						break;
				}
				videoSiguiente = 0;
				mapas.deleteAllElements();
				itemsInventario.removeAllElements();
				videosFase.deleteAllElements();
				converFase.deleteAllElements();
				midlet.preguardado.removeAllElements();
				parteJuego = -1;
				actual = -1;
				redibujar = true;
				System.gc();
				break;
			case Utils.reMensaje:
				ocupado = true;
				mostrarMensaje = true;
				redibujar = true;
				Utils.mensaje.delete(0, Utils.mensaje.length());
				Utils.mensaje.append(aux.info);
				break;
			case Utils.reAddInvent:
				itemsInventario.addElement(buscaItem(Integer.parseInt(aux.info)));
				if (!Utils.cargandoPartida) {
					mostrarInventario = true;
					parteJuego = juego;
				}
				redibujar = true;
				invSelec = itemsInventario.size() - 1;
				midlet.pregarduar(aux);
				break;
			case Utils.reAddInventSin:
				itemsInventario.addElement(buscaItem(Integer.parseInt(aux.info)));
				invSelec = itemsInventario.size() - 1;
				midlet.pregarduar(aux);
				break;
			case Utils.reDelInvent:
				Item auxItem;
				int id = Integer.parseInt(aux.info);
				int size = itemsInventario.size();
				for (int i = 0; i < size; i++) {
					auxItem = (Item) itemsInventario.elementAt(i);
					if (auxItem.id == id)
						itemsInventario.removeElementAt(i);
					i = size;
				}
				midlet.pregarduar(aux);
				break;
			case Utils.reVideo:
				videoSiguiente = Integer.parseInt(aux.info);
				parteJuego = video;
				ocupado = true;
				redibujar = true;
				break;
			case Utils.reIrEstancia:
				id = Integer.parseInt(aux.info);
				((Mapa) mapas.search(mapaActual).data).activa = id;
				escenActual = ((Mapa) mapas.search(mapaActual).data)
						.getEscenActual();
				parteJuego = juego;
				//no lo pongo ocupado, ya que se supone que tiene carcter de
				//ultima accin
				break;
			case Utils.reCamConsecuCon:
				int fin = 0;
				int inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				id = Integer.parseInt(aux.info.substring(inicio, fin));
				inicio = fin + 1;
				fin++;
				//identificador de la opcin seleccionada
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				String idOp = aux.info.substring(inicio, fin);
				inicio = fin + 1;
				fin++;
				Conversacion auxCon = (Conversacion) converFase.search(id).data;
				if (auxCon.id == id)
					auxCon.setConseActivaAcc(idOp, Integer.parseInt(aux.info
							.substring(inicio)));
				else {
					AVLTree arbol = converFase;
					auxCon = (Conversacion) arbol.search(id).data;
					if (auxCon != null)
						auxCon.setConseActivaAcc(idOp, Integer.parseInt(aux.info
								.substring(inicio)));
	
				}
				midlet.pregarduar(aux);
				break;
			case Utils.reCamConsecuIt:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxItem = buscaItem(Integer.parseInt(aux.info
						.substring(inicio, fin)));
				inicio = fin + 1;
				fin++;
				//				identificador de la opcin seleccionada
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				idOp = aux.info.substring(inicio, fin);
				inicio = fin + 1;
				fin++;
				auxItem.setConseActivaAcc(idOp, Integer.parseInt(aux.info
						.substring(inicio)));
				midlet.pregarduar(aux);
				break;
			case Utils.reCamConsecuEs:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				Estancia auxEstancia = buscaEstancia(Integer.parseInt(aux.info
						.substring(inicio, fin)));
				inicio = fin + 1;
				fin++;
				//				identificador de la opcin seleccionada
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				idOp = aux.info.substring(inicio, fin);
				inicio = fin + 1;
				fin++;
				auxEstancia.setConseActivaAcc(idOp, Integer.parseInt(aux.info
						.substring(inicio)));
				midlet.pregarduar(aux);
				break;
			case Utils.reCamConsecuCom:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxItem = buscaItem(Integer.parseInt(aux.info
						.substring(inicio, fin)));
				inicio = fin + 1;
				fin++;
				//identificador de la combinacin que se busca
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				int idCom = Integer.parseInt(aux.info.substring(inicio, fin));
				inicio = fin + 1;
				fin++;
				auxItem.setConseActivaCom(idCom, Integer.parseInt(aux.info
						.substring(inicio)));
				midlet.pregarduar(aux);
				break;
			case Utils.reMapa:
				parteJuego = mapa;
				break;
			case Utils.reCamMapa:
				mapaActual = Integer.parseInt(aux.info);
				if (actual == mapa) {
					//Le digo que el actual es video, ya que como actual es mapa,
					// pues
					//al entrar en drawScreen, recargara el escenario
					actual = video;
					redibujar = true;
				}
				break;
			case Utils.reConversacion:
				converSelec = Integer.parseInt(aux.info);
				parteJuego = conversacion;
				redibujar = true;
				break;
			case Utils.reRespuesta:
				Utils.mensaje.delete(0, Utils.mensaje.length());
				Utils.mensaje.append(aux.info);
				habParti = true;
				ocupado = true;
				break;
			case Utils.reRespuOscar:
				Utils.mensaje.delete(0, Utils.mensaje.length());
				Utils.mensaje.append(aux.info);
				ocupado = true;
				break;
			case Utils.reJuego:
				parteJuego = juego;
				break;
			case Utils.reCargaOpSup:
				auxCon = ((Conversacion) seleccionado);
				longIdOp = auxCon.opciones[opcionSele].id.length()
						- Integer.parseInt(aux.info);
				iniIdOp.delete(0, iniIdOp.length());
				iniIdOp.append(auxCon.opciones[opcionSele].id.substring(0,
						longIdOp - 1));
				break;
			case Utils.reCargaOpInf:
				auxCon = ((Conversacion) seleccionado);
				longIdOp = auxCon.opciones[opcionSele].id.length() + 1;
				iniIdOp.delete(0, iniIdOp.length());
				iniIdOp.append(auxCon.opciones[opcionSele].id);
				break;
			case Utils.reQuitarOp:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxCon = (Conversacion) converFase.search(Integer.parseInt(aux.info
						.substring(inicio, fin))).data;
				inicio = fin + 1;
				auxCon.borrarOp(aux.info.substring(inicio));
				midlet.pregarduar(aux);
				break;
			case Utils.reQuitarOpYSup:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxCon = (Conversacion) converFase.search(Integer.parseInt(aux.info
						.substring(inicio, fin))).data;
				inicio = fin + 1;
				fin++;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				Opcion auxOp = auxCon.getOpcion(aux.info.substring(inicio, fin));
				inicio = fin + 1;
				//Contador del numero de opciones inferiores, para una determinada
				// opcin
				int contador = 0;
				longIdOp = auxOp.id.length()
						- Integer.parseInt(aux.info.substring(inicio));
				iniIdOp.delete(0, iniIdOp.length());
				iniIdOp.append(auxOp.id.substring(0, longIdOp));
				for (int i = 0; i < auxCon.opciones.length; i++) {
					if (auxCon.opciones[i] != null) {
						if ((auxCon.opciones[i].id.substring(0, longIdOp)
								.equals(iniIdOp.toString()))
								&& auxCon.opciones[i].id.length() > longIdOp) {
							contador++;
						}
					}
				}
				//Si no hay mas inferiores, aparte de la de volver a las
				// principales...
				if (contador <= 1) {
					//Ponemos la Principal y la actual a null
					for (int i = 0; i < auxCon.opciones.length; i++) {
						if (auxCon.opciones[i] != null) {
							if ((auxCon.opciones[i].id.substring(0, longIdOp)
									.equals(iniIdOp.toString()))) {
								//								//la opcin seleccionada ser lo ultimo en
								// borrar, porque sino mas abajo nos dara
								//problemas con null
								if (auxCon.opciones[i].id != auxOp.id)
									auxCon.opciones[i] = null;
							}
						}
					}
				}
				//				Contador del numero de opciones superiores, para una determinada
				// opcin
				//esto es necesario ya que el cargaOpSup viene implcito en este
				// resultado
				//y si resulta que al eliminar al sup, el nivel superior se queda
				// sin ninguna
				//opcin, entonces deberemos ir otro nivel por encima.
				int temp = contador;
				contador = 0;
				for (int i = 0; i < auxCon.opciones.length; i++) {
					if (auxCon.opciones[i] != null) {
						if ((auxCon.opciones[i].id.length() == longIdOp)) {
							contador++;
						}
					}
				}
				if (contador == 0) {
					//Si no hay ninguna inmediatamente superior a la actual,
					// tendremos
					//que ir a orta 2 niveles por encima, ej del grupo
					//de opciones aba,abb ... , al grupo de opciones a, b...
					longIdOp = auxOp.id.length() - 2;
					iniIdOp.delete(0, iniIdOp.length());
					iniIdOp.append(auxOp.id.substring(0, longIdOp));
				}
				if (temp <= 1)
					auxCon.borrarOp(auxOp.id);
				midlet.pregarduar(aux);
				break;
			case Utils.reCamActivaOpIt:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxItem = buscaItem(Integer.parseInt(aux.info
						.substring(inicio, fin)));
				inicio = fin + 1;
				auxOp = auxItem.getOpcion(aux.info.substring(inicio));
				if (auxOp.activa)
					auxOp.activa = false;
				else
					auxOp.activa = true;
				midlet.pregarduar(aux);
				break;
			case Utils.reCamActivaOpEs:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxEstancia = buscaEstancia(Integer.parseInt(aux.info.substring(
						inicio, fin)));
				inicio = fin + 1;
				auxOp = auxEstancia.getOpcion(aux.info.substring(inicio));
				if (auxOp.activa)
					auxOp.activa = false;
				else
					auxOp.activa = true;
				midlet.pregarduar(aux);
				break;
			case Utils.reCamActivaOpCon:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				id = Integer.parseInt(aux.info.substring(inicio, fin));
				inicio = fin + 1;
				idOp = aux.info.substring(inicio);
	
				auxCon = (Conversacion) converFase.search(id).data;
				if (auxCon.id == id) {
					auxOp = auxCon.getOpcion(idOp);
					if (auxOp.activa)
						auxOp.activa = false;
					else
						auxOp.activa = true;
				} else {
					AVLTree arbol = converFase;
					auxCon = (Conversacion) arbol.search(id).data;
					if (auxCon != null) {
						auxOp = auxCon.getOpcion(idOp);
						if (auxOp.activa)
							auxOp.activa = false;
						else
							auxOp.activa = true;
					}
				}
				midlet.pregarduar(aux);
				break;
			case Utils.reCaminar:
				if (buscarRuta())
					caminando = true;
				break;
			case Utils.reCamActivaCom:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxItem = buscaItem(Integer.parseInt(aux.info
						.substring(inicio, fin)));
				inicio = fin + 1;
				idCom = Integer.parseInt(aux.info.substring(inicio));
				Combinacion auxCom = (Combinacion) auxItem.combinaciones
						.search(idCom).data;
				if (auxCom.activa == false)
					auxCom.activa = true;
				else
					auxCom.activa = false;
				midlet.pregarduar(aux);
				break;
			case Utils.reEliminarItem:
				boolean eliminado = false;
				id = Integer.parseInt(aux.info);
				//Si eliminamos un Item sin antes descargarlo del escenario,
				//el Item aparecer en todos los escenarios, formar
				//parte de LM para siempre
				if (actual == juego)
					descargarEscenario();
	
				//haya el escenario al que pertenece el Item
	
				//1 miramos si es un Item perteneciente al escenario actual
				if (buscaItemEsc(escenActual, id, true) != null) {
					escenActual.items.deleteKey(id);
					eliminado = true;
				}
	
				if (!eliminado) {
					//2 miramos si es un Item perteneciente a la estancia actual.
					Node auxN = ((Mapa) mapas.search(mapaActual).data)
							.getEstanActual().escenarios.root;
					Escenario auxEscenario;
					((Mapa) mapas.search(mapaActual).data).getEstanActual().escenarios
							.inorden(auxN);
					auxN = ((Mapa) mapas.search(mapaActual).data).getEstanActual().escenarios
							.next();
					while (auxN != null && eliminado == false) {
						auxEscenario = (Escenario) auxN.data;
						//Si el escenario no es el actual, que ya comprobamos
						// antes...
						if (auxEscenario.id != escenActual.id) {
							if (buscaItemEsc(auxEscenario, id, true) != null) {
								auxEscenario.items.deleteKey(id);
								eliminado = true;
							}
						}
						auxN = ((Mapa) mapas.search(mapaActual).data)
								.getEstanActual().escenarios.next();
					}
				}
	
				if (!eliminado) {
					//3 Miramos si es un Item perteneciente a el mapa actual.
					Node auxN = ((Mapa) mapas.search(mapaActual).data).estancias.root;
					((Mapa) mapas.search(mapaActual).data).estancias.inorden(auxN);
					auxN = ((Mapa) mapas.search(mapaActual).data).estancias.next();
					while (auxN != null && eliminado == false) {
						auxEstancia = (Estancia) auxN.data;
						//					Si la estancia no es la actual, que ya comprobamos
						// antes...
						if (auxEstancia.id != ((Mapa) mapas.search(mapaActual).data)
								.getEstanActual().id) {
							Node auxN2 = auxEstancia.escenarios.root;
							auxEstancia.escenarios.inorden(auxN2);
							auxN2 = auxEstancia.escenarios.next();
							while (auxN2 != null) {
								Escenario auxEscenario = (Escenario) auxN2.data;
								if (buscaItemEsc(auxEscenario, id, true) != null) {
									auxEscenario.items.deleteKey(id);
									eliminado = true;
								}
								auxN2 = auxEstancia.escenarios.next();
							}
						}
						auxN = ((Mapa) mapas.search(mapaActual).data).estancias
								.next();
					}
				}
	
				if (!eliminado) {
					//Por ltimo miramos si es de otro mapa.
					Node auxN = mapas.root;
					Mapa auxMapa;
					mapas.inorden(auxN);
					auxN = mapas.next();
					while (auxN != null && eliminado == false) {
						auxMapa = (Mapa) auxN.data;
						if (auxMapa.id != ((Mapa) mapas.search(mapaActual).data).id) {
							Node auxN2 = auxMapa.estancias.root;
							auxMapa.estancias.inorden(auxN2);
							auxN2 = auxMapa.estancias.next();
							while (auxN2 != null) {
								auxEstancia = (Estancia) auxN2.data;
								Node auxN3 = auxEstancia.escenarios.root;
								auxEstancia.escenarios.inorden(auxN3);
								auxN3 = auxEstancia.escenarios.next();
								while (auxN3 != null) {
									Escenario auxEscenario = (Escenario) auxN3.data;
									if (buscaItemEsc(auxEscenario, id, true) != null) {
										auxEscenario.items.deleteKey(id);
										eliminado = true;
									}
									auxN3 = auxEstancia.escenarios.next();
								}
								auxN2 = auxMapa.estancias.next();
							}
						}
						auxN = mapas.next();
					}
				}
	
				size = itemsInventario.size();
				//Y tambin lo borramos del inventario
				for (int i = 0; i < size; i++) {
					auxItem = (Item) itemsInventario.elementAt(i);
					if (auxItem.id == id) {
						itemsInventario.removeElementAt(i);
						i = size;//salimos del for
					}
				}
				//Una vez borrado el Item, volvemos a cargar el escenario.
				if (actual == juego)
					cargarEscenario();
				midlet.pregarduar(aux);
				break;
			case Utils.reEliminarEstan:
				eliminado = false;
				id = Integer.parseInt(aux.info);
				//Si eliminamos una estancia sin antes descargarlo el mapa,
				//la estancia aparecer en todos los mapa, formar
				//parte de LM para siempre
				if (actual == mapa)
					descargarMapa();
	
				//haya el mapa al que pertenece la estancia
	
				//1 Miramos si es la estancia actual
				auxEstancia = (Estancia) seleccionado;
				if (auxEstancia.id == id) {
					((Mapa) mapas.search(mapaActual).data).estancias.deleteKey(id);
					eliminado = true;
				}
	
				if (!eliminado) {
					//2 Miramos si es una Estancia perteneciente a el mapa actual.
					Node auxN = ((Mapa) mapas.search(mapaActual).data).estancias.root;
					((Mapa) mapas.search(mapaActual).data).estancias.inorden(auxN);
					auxN = ((Mapa) mapas.search(mapaActual).data).estancias.next();
					while (auxN != null && eliminado == false) {
						auxEstancia = (Estancia) auxN.data;
						//					Si la estancia no es la actual, que ya comprobamos
						// antes...
						if (auxEstancia.id != ((Mapa) mapas.search(mapaActual).data)
								.getEstanActual().id) {
							if (auxEstancia.id == id) {
								((Mapa) mapas.search(mapaActual).data).estancias
										.delete(auxN);
								eliminado = true;
							}
						}
						auxN = ((Mapa) mapas.search(mapaActual).data).estancias
								.next();
					}
				}
	
				if (!eliminado) {
					//Por ltimo miramos si es de otro mapa.
					Node auxN = mapas.root;
					mapas.inorden(auxN);
					auxN = mapas.next();
					while (auxN != null && eliminado == false) {
						Mapa auxMapa = (Mapa) auxN.data;
						if (auxMapa.id != ((Mapa) mapas.search(mapaActual).data).id) {
							Node auxN2 = auxMapa.estancias.root;
							auxMapa.estancias.inorden(auxN2);
							auxN2 = auxMapa.estancias.next();
							while (auxN2 != null) {
								auxEstancia = (Estancia) auxN2.data;
								if (auxEstancia.id == id) {
									auxMapa.estancias.delete(auxN2);
									eliminado = true;
								}
								auxN2 = auxMapa.estancias.next();
							}
						}
						auxN = mapas.next();
					}
				}
	
				//Una vez borrada la estancia, volvemos a cargar el mapa.
				if (actual == mapa)
					cargarMapa();
				midlet.pregarduar(aux);
				break;
			case Utils.reSalirJuego:
				midlet.salir();
				break;
			case Utils.reCambiaEscenario:
				id = Integer.parseInt(aux.info);
				descargarEscenario();
				((Mapa) mapas.search(mapaActual).data).getEstanActual().actual = id;
				escenActual = ((Mapa) mapas.search(mapaActual).data)
						.getEscenActual();
				//Le digo que el actual es video, ya que como parteJuego es juego,
				// pues
				//al entrar en drawScreen, recargara el escenario
				actual = video;
				redibujar = true;
				break;
			case Utils.reCamNombreIt:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxItem = buscaItem(Integer.parseInt(aux.info
						.substring(inicio, fin)));
				inicio = fin + 1;
				auxItem.nombre = aux.info.substring(inicio);
				letra = -6;
				midlet.pregarduar(aux);
				break;
			case Utils.reCamNombreEs:
				fin = 0;
				inicio = 0;
				while (aux.info.charAt(fin) != ';') {
					fin++;
				}
				auxEstancia = buscaEstancia(Integer.parseInt(aux.info.substring(
						inicio, fin)));
				inicio = fin + 1;
				auxEstancia.nombre = aux.info.substring(inicio);
				letra = -6;
				midlet.pregarduar(aux);
				break;
			case Utils.reCaminarBlo:
				if (buscarRuta()) {
					caminando = true;
					ocupado = true;
				}
				break;
			case Utils.reCambiaVisibleIt:
				auxItem = buscaItem(Integer.parseInt(aux.info));
				if (auxItem.isVisible())
					auxItem.setVisible(false);
				else
					auxItem.setVisible(true);
				midlet.pregarduar(aux);
				break;
			case Utils.reCambiaVisibleEs:
				auxEstancia = buscaEstancia(Integer.parseInt(aux.info));
				if (auxEstancia.representa.isVisible())
					auxEstancia.representa.setVisible(false);
				else
					auxEstancia.representa.setVisible(true);
				midlet.pregarduar(aux);
				break;
			case Utils.reCombinando:
				combinando = true;
				combinado = (Item) seleccionado;
				break;
			case Utils.reCamApariencia:
				Utils.actualizarApariencia(Integer.parseInt(aux.info));
				Utils.cargarImagen(Utils.spOscarFren);
				oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
				oscar.setFrameSequence(Utils.getFrames(Utils.spOscarFren));
				midlet.pregarduar(aux);
				redibujar=true;
				break;
		}
		resultados.removeElementAt(0);
		if (resultados.size() == 0) {
			opcionSele = 0;
			Utils.cargandoPartida = false;
			if (intentaCombinar) {
				combinando = false;
				intentaCombinar = false;
				combinado = null;
				letra = -6;
			}
		}
		System.gc();
	}

	/**
	 * Muestra las opciones del objeto seleccionado en la pantalla de juego
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getOpciones(Graphics g) {
		Utils.numOps = 0;
		Utils.spacing = Utils.lowFont.getHeight();
		for (int i = 0; i < seleccionado.longAcciones(); i++)
			if (seleccionado.getAccionActiva(i))
				Utils.numOps++;
		if (Utils.grupo == 0) {//grupo pequeo
			altoOpciones = Utils.lowFont.getHeight() * (Utils.numOps);
			Utils.spacing = Utils.lowFont.getHeight();
		} else {
			altoOpciones = Utils.highFont.getHeight() * (Utils.numOps);
			Utils.spacing = Utils.highFont.getHeight();
		}
		//Dibujamos el cuadrado gris
		g.setColor(0x00DDDDDD);
		int tempX = Utils.inicioPantallaX + Utils.getTamGrupoX()
				- anchoOpciones;
		int tempY = Utils.inicioPantallaY + Utils.getTamGrupoY()
				- Utils.getSpaceTextItem() - altoOpciones;
		g.fillRect(tempX, tempY, anchoOpciones, altoOpciones);
		g.setColor(Utils.highColor);
		//Ahora la flechita
		g.fillTriangle(tempX, tempY + (opcionSele * Utils.spacing)
				+ Utils.spacing, tempX + 7, tempY
				+ (opcionSele * Utils.spacing) + (Utils.spacing / 2), tempX,
				tempY + (opcionSele * Utils.spacing));
		//Guardamos los valores de startWidth y height en x e y
		int aux;
		aux = tempX;
		tempX = Utils.startWidth;
		Utils.startWidth = aux + ((anchoOpciones * 25) / 100);
		aux = tempY;
		tempY = Utils.startHeight;
		Utils.startHeight = aux;
		//Escribimos las opciones
		g.setColor(Utils.highColor);
		int cuentaActiva = 0;

		for (int i = 0; i < seleccionado.longAcciones(); i++) {
			if (seleccionado.getAccionActiva(i) == true) {
				g.drawString(seleccionado.getNombreAccion(i), Utils.startWidth,
						Utils.startHeight + (cuentaActiva * Utils.spacing),
						Graphics.TOP | Graphics.LEFT);
				cuentaActiva++;
			}
		}

		//Restablecemos el valor de StartWidth y StartHeight y spacing
		Utils.startWidth = tempX;
		Utils.startHeight = tempY;
		Utils.spacing = (Utils.lowFont.getHeight() * 85) / 100;

	}

	/**
	 * Muestra Un mensaje en la pantalla de juego
	 * 
	 * @param g
	 *            objeto Graphics
	 */
	public void getMensaje(Graphics g) {
		//Dibujamos el cuadrado gris
		g.setColor(0x00DDDDDD);
		int tempX = Utils.inicioPantallaX
				+ ((Utils.getTamGrupoX() - anchoMensaje) / 2);
		int tempY = Utils.inicioPantallaY
				+ ((Utils.getTamGrupoY() - Utils.getSpaceTextItem()
						- altoInventario - altoMensaje) / 2);
		g.fillRect(tempX, tempY, anchoMensaje, altoMensaje);

		g.setColor(Utils.highColor);
		g.setFont(Utils.lowFont);
		//Guardamos los valores de startWidth, startHeight
		//numLetras y numLineas
		int nAux;
		nAux = tempX;
		tempX = Utils.startWidth;
		Utils.startWidth = nAux + Utils.lowFont.getSize();
		nAux = tempY;
		tempY = Utils.startHeight;
		Utils.startHeight = nAux;
		nAux = Utils.spaceMen;
		int nAux2 = Utils.numLineas;

		//Modificamos algunos valores necesarios para la escritura
		Utils.spaceMen = anchoMensaje;
		switch (Utils.grupo) {
		case 0://128x128
			Utils.numLineas = altoMensaje / Utils.lowFont.getHeight() - 1;
			break;
		case 1://176x200
			Utils.numLineas = altoMensaje / Utils.lowFont.getHeight();
			break;
		}

		//Imprimimos el mensaje
		mostrarMensaje(Utils.mensaje.toString(), "CERRAR", g);

		//Restablecemos el valor de StartWidth y StartHeight
		Utils.startWidth = tempX;
		Utils.startHeight = tempY;

		if (Utils.grupo == 1)//Si el movil es de grupo grande
			g.setFont(Utils.highFont);
	}

	/**
	 * Reajusta algunas variables como son el ancho y alto de la pantalla. Carga
	 * el numero de letras que caben en el ancho de la pantalla, y el numero de
	 * lneas que caben en el alto de la pantalla
	 *  
	 */
	public void ajustarVariables() {
		setFullScreenMode(true);
		irMenu = false;
		//ancho y alto de la pantalla
		Utils.width = getWidth();
		Utils.height = getHeight();
		//Espacio entre cada una de las lneas de la vieta
		Utils.spacing = (Utils.lowFont.getHeight() * 85) / 100;
		if (parteJuego == juego || parteJuego == mapa) {

			//Calcula la altura a la que se mostrar el nombre de los items
			Utils.startHeight = (Utils.height / 2) - (Utils.getTamGrupoY() / 2)
					+ (Utils.getTamGrupoY() - Utils.getSpaceTextItem());

			Utils.startWidth = (Utils.width / 2) - (Utils.getTamGrupoX() / 2)
					+ Utils.lowFont.getSize();
		} else if (parteJuego == video) {//estara en Video
			//espacio que dejaremos para el texto de la vieta
			int espacioTexto = (Utils.getTamGrupoY() * 33) / 100;
			Utils.numLineas = espacioTexto / Utils.lowFont.getHeight() - 1;
			Utils.startHeight = (Utils.height / 2) - (Utils.getTamGrupoY() / 2);
			Utils.startWidth = (Utils.width / 2) - (Utils.getTamGrupoX() / 2)
					+ Utils.lowFont.getSize();
		} else if (parteJuego == conversacion) {
			Utils.startHeight = Utils.inicioPantallaY + oscar.getHeight();
			Utils.numLineas = (Utils.getTamGrupoY() - oscar.getHeight())
					/ Utils.lowFont.getHeight() - 1;
			if (mostrarOpciones) {
				Utils.startWidth = Utils.inicioPantallaX
						+ ((oscar.getWidth() * 32) / 100);
				Utils.spaceMen = Utils.getTamGrupoX()
						- ((oscar.getWidth() * 60) / 100);
			} else {
				Utils.startWidth = Utils.inicioPantallaX;
				Utils.spaceMen = Utils.getTamGrupoX();
			}
		}

	}

	/**
	 * Devuelve el entero que identifica a una parte del juego
	 * 
	 * @param parte
	 *            Parte del juego de la que quiero saber el identificador
	 * @return el identificador de la parte del juego
	 */
	private int getParte(String parte) {
		if (parte.equals("video"))
			return video;
		if (parte.equals("juego"))
			return juego;
		if (parte.equals("mapa"))
			return mapa;
		return -1;
	}

	/**
	 * Haya el valor hexadecimal de un determinado color.
	 * 
	 * @param color
	 *            Color del que queremos saber su valor hexadecimal
	 * @return el valor en hexadecimal de un determinado color
	 */
	private int getColorHex(String color) {
		if (color.equals("Rojo"))
			return 0xFF0000;
		if (color.equals("Verde"))
			return 0x009900;
		if (color.equals("Crema"))
			return 0xFFFF99;
		if (color.equals("Amarillo"))
			return 0xD1940C;
		if (color.equals("Marrn"))
			return 0x663300;

		if (color.equals("Azul"))
			return 0x003366;
		if (color.equals("Negro"))
			return 0x000000;
		if (color.equals("Naranja"))
			return 0xFF6600;
		if (color.equals("Rosa"))
			return 0xFF00FF;
		return -1;
	}

	/**
	 * Imprime un mensaje por pantalla
	 * 
	 * @param mensaje
	 *            Mensaje a imprimir
	 * @param finMensaje
	 *            Cadena de texto a poner al final del mensaje para indicar que
	 *            el mensaje se ha terminado
	 * @param g
	 *            Objeto Graphics.
	 */
	public static void mostrarMensaje(String mensaje, String finMensaje,
			Graphics g) {
		System.gc();
		//		inicializamos el valor de mensaje, y palabraMensaje
		Utils.mensaje.delete(0, Utils.mensaje.length());
		Utils.palabrasMensaje.removeAllElements();
		//Metemos el texto
		Utils.mensaje.append(mensaje);
		//Hayamos cada una de las palabras del mensaje
		StringBuffer aux = new StringBuffer();
		char temp;
		for (int i = 0; i < Utils.mensaje.length(); i++) {
			temp = Utils.mensaje.charAt(i);
			if (temp != ' ')
				aux.append(temp);
			else {
				Utils.palabrasMensaje.addElement(aux.toString());
				aux.delete(0, aux.length());
			}
			if (i + 1 == Utils.mensaje.length()) {
				Utils.palabrasMensaje.addElement(aux.toString());
				aux.delete(0, aux.length());
			}
		}

		//		aux2 almacena el numero de letras que lleva una lnea
		int aux2 = 0;
		//Identifica en que lnea toca imprimir
		int lineaAux = 0;
		int size = Utils.palabrasMensaje.size();
		int tamPalabra = 0;
		String palabra;
		for (int i = 0; i < size; i++) {
			palabra = ((String) Utils.palabrasMensaje.elementAt(i));
			tamPalabra = palabra.length() * Utils.widthLF;
			//la ideal sera la de abajo, pero como carga demasiado pues uso la
			// anterior
			//tamPalabra=Utils.lowFont.stringWidth(palabra);
			//Si es salto de lnea, imprimimos y aumentamos en una la lineaAux
			if (palabra.equals("\\n")) {
				if (lineaAux >= Utils.linea
						&& lineaAux <= Utils.numLineas + Utils.linea)
					g
							.drawString(
									aux.toString(),
									Utils.startWidth,
									Utils.startHeight
											+ ((lineaAux - Utils.linea) * Utils.spacing),
									Graphics.TOP | Graphics.LEFT);
				else if (lineaAux > Utils.numLineas + Utils.linea)
					size = 0;//para salir del bucle
				aux.delete(0, aux.length());
				//pasamos a la linea siguiente y dejamos un hueco en blanco
				// entre medias.
				lineaAux = lineaAux + 2;
				aux2 = 0;
			} else {
				if (tamPalabra > Utils.spaceMen) {
					if (lineaAux >= Utils.linea
							&& lineaAux <= Utils.numLineas + Utils.linea)
						//					imprimimos lo que hay previamente
						g
								.drawString(
										aux.toString(),
										Utils.startWidth,
										Utils.startHeight
												+ ((lineaAux - Utils.linea) * Utils.spacing),
										Graphics.TOP | Graphics.LEFT);
					aux.delete(0, aux.length());
					lineaAux++;
					aux2 = 0;
					//Imprimimos la primera parte
					int noCaben = (tamPalabra - Utils.spaceMen) / Utils.widthLF;
					aux.append(palabra.substring(0, palabra.length() - noCaben
							- 1));
					if (lineaAux >= Utils.linea
							&& lineaAux <= Utils.numLineas + Utils.linea)
						g
								.drawString(
										aux.toString(),
										Utils.startWidth,
										Utils.startHeight
												+ ((lineaAux - Utils.linea) * Utils.spacing),
										Graphics.TOP | Graphics.LEFT);
					aux.delete(0, aux.length());
					lineaAux++;
					aux2 = 0;
					//					Y luego el final que no entraba lo aadimos a la palabra
					//de la siguiente lnea
					aux.append(palabra
							.substring(palabra.length() - noCaben - 1)
							+ " ");
					aux2 = aux2
							+ palabra.substring(palabra.length() - noCaben - 1)
									.length() * Utils.widthLF
							+ Utils.lowFont.charWidth(' ');
				} else {
					//si el numero de letras de esa lnea (aux2) es menor que
					// el numero
					//de letras que caben en esta pantalla, restndole adems
					//el tamao de la ultima palabra,
					if (aux2 + tamPalabra < Utils.spaceMen) {
						//Introducimos en aux, la palabra seguida de un espacio
						aux.append(palabra + " ");
						//actualizamos el valor de aux2
						aux2 = aux2 + tamPalabra + Utils.lowFont.charWidth(' ');
					} else {
						//Solo imprime las filas que sern visibles, es decir
						//para no imprimir lneas que no se veran pues estara
						//en la parte negra
						if (lineaAux >= Utils.linea
								&& lineaAux <= Utils.numLineas + Utils.linea)
							g
									.drawString(
											aux.toString(),
											Utils.startWidth,
											Utils.startHeight
													+ ((lineaAux - Utils.linea) * Utils.spacing),
											Graphics.TOP | Graphics.LEFT);
						else if (lineaAux > Utils.numLineas + Utils.linea)
							size = 0;//para salir del bucle
						//al imprimir una lnea borramos el contenido de aux
						aux.delete(0, aux.length());
						//aumentamos el valor de lineaAux
						lineaAux++;
						//aux2 vuelve a ser 0
						aux2 = 0;
						//y en aux metemos la palabra que no se ha introducido
						//en la lnea que se acaba de imprimir, porque el
						// numeroLetras
						//de esa lnea ya sobrepasaba los limites
						aux.append(palabra + " ");
						aux2 = aux2 + tamPalabra + Utils.lowFont.charWidth(' ');

					}
				}
			}
			//Cuando llegamos al final, imprimimos la lnea (este como este)
			if (i + 1 == Utils.palabrasMensaje.size()) {
				if (lineaAux >= Utils.linea
						&& lineaAux <= Utils.numLineas + Utils.linea)
					g
							.drawString(
									aux.toString(),
									Utils.startWidth,
									Utils.startHeight
											+ ((lineaAux - Utils.linea) * Utils.spacing),
									Graphics.TOP | Graphics.LEFT);
			}
		}
		//		el numero de lneas que ocupa el mensaje.
		Utils.numLineasMensaje = lineaAux;
		//si el numero de lneas que caben en la pantalla + la lnea
		//en la cual se imprime la primera lnea es menor que el numero
		//de lneas que ocupa el mensaje (teniendo en cuenta que si el
		//mensaje ocupa 7 lneas, numLineas valdr 6), se pinta una flecha
		//hacia abajo
		if (Utils.numLineas + Utils.linea < Utils.numLineasMensaje) {
			g.setColor(0x00000000);
			if (Utils.grupo == 1)//Tamao grande
				g.fillTriangle((Utils.width / 2) - 7, Utils.startHeight
						+ (Utils.numLineas * Utils.spacing) + Utils.spacing
						+ (Utils.spacing / 4), (Utils.width / 2),
						Utils.startHeight + (Utils.numLineas * Utils.spacing)
								+ Utils.spacing + (Utils.spacing / 4) + 7,
						(Utils.width / 2) + 7, Utils.startHeight
								+ (Utils.numLineas * Utils.spacing)
								+ Utils.spacing + (Utils.spacing / 4));
			else
				g.fillTriangle((Utils.width / 2) - 7, Utils.startHeight
						+ (Utils.numLineas * Utils.spacing) + Utils.spacing
						+ (Utils.spacing / 2), (Utils.width / 2),
						Utils.startHeight + (Utils.numLineas * Utils.spacing)
								+ Utils.spacing + (Utils.spacing / 2) + 7,
						(Utils.width / 2) + 7, Utils.startHeight
								+ (Utils.numLineas * Utils.spacing)
								+ Utils.spacing + (Utils.spacing / 2));
		} else {
			g.drawString(finMensaje, (Utils.width - Utils.lowFont
					.stringWidth(finMensaje)) / 2, Utils.startHeight
					+ (Utils.numLineas * Utils.spacing) + 2 * Utils.spacing
					- Utils.lowFont.getHeight(), Graphics.TOP | Graphics.LEFT);
		}
	}

	/**
	 * Imprime un mensaje en la lnea de textos
	 * 
	 * @param g
	 *            objeto Graphics
	 * @param texto
	 *            el mensaje a mostrar.
	 */
	private void getLineatexto(Graphics g, String texto) {
		if (combinando)
			texto = "USAR " + combinado.nombre + " CON " + texto;
		int tamPalabra = texto.length() * Utils.widthLF;

		//la ideal sera la de abajo, pero como carga demasiado pues uso la
		// anterior
		//tamPalabra=Utils.lowFont.stringWidth(texto);
		if (tamPalabra > disponible) {
			if (letra < texto.length()) {
				if (tmpAcumulado % 80 == 0)
					letra++;
			} else
				letra = -6;

			redibujar = true;
		}
		if (letra < 0) {
			if (texto.length() >= numLetras - 1)
				g.drawString(texto.substring(0, numLetras - 1),
						Utils.startWidth, Utils.startHeight, Graphics.TOP
								| Graphics.LEFT);
			else
				g.drawString(texto, Utils.startWidth, Utils.startHeight,
						Graphics.TOP | Graphics.LEFT);
		} else {
			if (texto.length() >= letra + numLetras - 1)
				g.drawString(texto.substring(letra, letra + numLetras - 1),
						Utils.startWidth, Utils.startHeight, Graphics.TOP
								| Graphics.LEFT);
			else
				g.drawString(texto.substring(letra), Utils.startWidth,
						Utils.startHeight, Graphics.TOP | Graphics.LEFT);
		}
	}

	/**
	 * Mueve la boca del personaje que esta hablando
	 */
	private void moverBoca() {
		if (!mostrarOpciones && tmpAcumulado % 200 == 0) {
			if (habParti)
				sprite.nextFrame();
			else
				oscar.nextFrame();
			redibujar = true;
		}
	}

	/**
	 * Cierra la boca del personaje que esta hablando
	 */
	private void cerrarBoca() {
		if (habParti)
			sprite.setFrame(0);
		else
			oscar.setFrame(0);
	}

	/**
	 * Se encarga de buscar la ruta adecuada, para llevar a nuestro persona
	 * desde su posicin actual, hasta el sitio de la pantalla en el que hayamos
	 * pulsado.
	 * 
	 * @return Boolean indicando si existe ruta hasta una determinada celda o
	 *         no.
	 */
	private boolean buscarRuta() {
		int celdaPunX = getCeldaX(puntero.getX() - Utils.inicioPantallaX
				+ (puntero.getWidth() / 2));
		int celdaPunY = getCeldaY(puntero.getY() - Utils.inicioPantallaY
				+ (puntero.getHeight() / 2));
		int celdaOscX = getCeldaX(oscar.getX() - Utils.inicioPantallaX
				+ (oscar.getWidth() - anchoCelda) / 2);
		int celdaOscY = getCeldaY(oscar.getY() - Utils.inicioPantallaY
				+ (oscar.getHeight()));

		if (buscaRuta.walkability[celdaPunX + (celdaPunY * buscaRuta.mapWidth)] == false) {
			while (buscaRuta.walkability[celdaPunX
					+ (celdaPunY * buscaRuta.mapWidth)] == false) {
				if (celdaPunY != celdaOscY) {
					if (celdaPunY < celdaOscY)
						celdaPunY = Math.min(celdaOscY, celdaPunY + 7);
					if (celdaPunY > celdaOscY)
						celdaPunY = Math.max(celdaOscY, celdaPunY - 7);

				} else {
					if (celdaPunX < celdaOscX)
						celdaPunX = Math.min(celdaOscX, celdaPunX + 7);
					if (celdaPunX > celdaOscX)
						celdaPunX = Math.max(celdaOscX, celdaPunX - 7);
				}
			}
		}
		//Solo busca una ruta si las celdas de inicio y destino son distintas.
		if (celdaPunX + (celdaPunY * buscaRuta.mapWidth) != celdaOscX
				+ (celdaOscY * buscaRuta.mapWidth)) {
			if (buscaRuta.findPath(celdaOscX, celdaOscY, celdaPunX, celdaPunY)) {
				pasoActual = 0;
				return true;
			}
		}
		return false;
	}

	/**
	 * Nos devuelve la posicin X de la celda en la que esta situada un
	 * determinado pxel
	 * 
	 * @param pixel
	 *            La posicin en pxeles de la que se quiere sabe a que celda
	 *            corresponde
	 * @return la posicin X de la celda a la que est apuntando el puntero.
	 */
	private int getCeldaX(int pixel) {
		return pixel / anchoCelda;
	}

	/**
	 * Nos devuelve la posicin Y de la celda en la que esta situada un
	 * determinado pxel
	 * 
	 * @param pixel
	 *            La posicin en pxeles de la que se quiere sabe a que celda
	 *            corresponde
	 * @return la posicin Y de la celda a la que est apuntando el puntero.
	 */
	private int getCeldaY(int pixel) {
		return pixel / altoCelda;
	}

	/**
	 * Nos devuelve la posicin X del pxel en el que esta situado una
	 * determinada celda
	 * 
	 * @param celda
	 *            La celda de la que se quiere sabe a que pxel corresponde
	 * @return el pxel X de la celda a la que est apuntando el puntero.
	 */
	private int getPixelX(int celda) {
		return Utils.inicioPantallaX + (celda * anchoCelda)
				- (oscar.getWidth() - anchoCelda) / 2;
	}

	/**
	 * Nos devuelve la posicin Y del pxel en el que esta situado una
	 * determinada celda
	 * 
	 * @param celda
	 *            La celda de la que se quiere sabe a que pxel corresponde
	 * @return el pxel Y de la celda a la que est apuntando el puntero.
	 */
	private int getPixelY(int celda) {
		return Utils.inicioPantallaY + (celda * altoCelda) - oscar.getHeight();
	}

	/**
	 * Haya la Estancia cuyo identificador coincide con el parmetro que le
	 * pasamos.
	 * 
	 * @param id
	 *            Identificador de la estancia a buscar
	 * @return La estancia buscado.
	 */
	private Estancia buscaEstancia(int id) {
		AVLTree arbol;
		Estancia auxEstancia;

		//1 Miramos si es la estancia actual
		if (seleccionado != null) {
			auxEstancia = (Estancia) seleccionado;
			if (auxEstancia.id == id)
				return auxEstancia;
		}

		//2 Miramos si es una Estancia perteneciente a el mapa actual.
		arbol = ((Mapa) mapas.search(mapaActual).data).estancias;
		auxEstancia = (Estancia) arbol.search(id).data;
		if (auxEstancia != null)
			return auxEstancia;

		//Por ltimo miramos si es de otro mapa.
		Node auxN = mapas.root;
		mapas.inorden(auxN);
		auxN = mapas.next();
		while (auxN != null) {
			Mapa auxMapa = (Mapa) auxN.data;
			if (auxMapa.id != ((Mapa) mapas.search(mapaActual).data).id) {
				arbol = ((Mapa) mapas.search(mapaActual).data).estancias;
				auxEstancia = (Estancia) arbol.search(id).data;
				if (auxEstancia != null)
					return auxEstancia;
			}
			auxN = mapas.next();
		}

		//Si no encuentra ninguna estancia con dicho identificador
		return null;
	}

	/**
	 * Haya el Item cuyo identificador coincide con el parmetro que le pasamos.
	 * 
	 * @param id
	 *            Identificador del Item a buscar
	 * @return El Item buscado.
	 */
	private Item buscaItem(int id) {
		Item auxItem;
		//Primero miramos si es el Item seleccionado.
		if (actual != conversacion && seleccionado != null) {
			auxItem = (Item) seleccionado;
			if (auxItem.id == id)
				return auxItem;
		}

		//2 miramos si es un Item perteneciente al escenario actual
		auxItem = buscaItemEsc(escenActual, id, false);
		if (auxItem != null)
			return auxItem;

		//3 miramos si es un Item perteneciente a la estancia actual.
		auxItem = buscaItemEst(((Mapa) mapas.search(mapaActual).data)
				.getEstanActual(), id);
		if (auxItem != null)
			return auxItem;

		//4 Miramos si es un Item perteneciente a el mapa actual.
		auxItem = buscaItemM((Mapa) mapas.search(mapaActual).data, id);
		if (auxItem != null)
			return auxItem;

		//Por ltimo miramos si es de otro mapa.
		auxItem = buscaItemTM(id);
		if (auxItem != null)
			return auxItem;

		//En caso de no encontrarlo, devuelve null
		return null;
	}

	/**
	 * Busca un Item en un escenario que se pasa por parmetro, cuyo
	 * identificador coincida con el que pasamos por parmetro
	 * 
	 * @param auxEscenario
	 *            Escenario donde buscar el Item
	 * @param id
	 *            Identificador del Item a buscar.
	 * @param comprueba
	 *            Indica se comprueba el Item seleccionado o no
	 * @return El item encontrado o null en caso de no encontrarlo.
	 */
	private Item buscaItemEsc(Escenario auxEscenario, int id, boolean comprueba) {
		Item auxItem;
		AVLTree arbol;
		Node auxN;

		if (comprueba && !mostrarInventario && actual != conversacion
				&& seleccionado != null) {
			auxItem = (Item) seleccionado;
			if (auxItem.id == id)
				return auxItem;
		}

		arbol = auxEscenario.items;
		auxN = arbol.search(id);
		if (auxN != null)
			return (Item) auxN.data;
		return null;
	}

	/**
	 * Busca un Item en un estancia que se pasa por parmetro, cuyo
	 * identificador coincida con el que pasamos por parmetro
	 * 
	 * @param auxEstancia
	 *            Estancia donde buscar el Item
	 * @param id
	 *            Identificador del Item a buscar.
	 * @return El item encontrado o null en caso de no encontrarlo.
	 */
	private Item buscaItemEst(Estancia auxEstancia, int id) {
		Node auxN = auxEstancia.escenarios.root;
		Escenario auxEscenario;
		auxEstancia.escenarios.inorden(auxN);
		auxN = auxEstancia.escenarios.next();
		while (auxN != null) {
			auxEscenario = (Escenario) auxN.data;
			//Si el escenario no es el actual, que ya comprobamos antes...
			if (auxEscenario.id != escenActual.id) {
				Item auxItem = buscaItemEsc(auxEscenario, id, false);
				if (auxItem != null)
					return auxItem;
			}
			auxN = auxEstancia.escenarios.next();
		}
		return null;
	}

	/**
	 * Busca un Item en un mapa que se pasa por parmetro, cuyo identificador
	 * coincida con el que pasamos por parmetro
	 * 
	 * @param auxMapa
	 *            Mapa donde buscar el Item
	 * @param id
	 *            Identificador del Item a buscar.
	 * @return El item encontrado o null en caso de no encontrarlo.
	 */
	private Item buscaItemM(Mapa auxMapa, int id) {
		Node auxN = auxMapa.estancias.root;
		Estancia auxEstancia;
		auxMapa.estancias.inorden(auxN);
		auxN = auxMapa.estancias.next();
		while (auxN != null) {
			auxEstancia = (Estancia) auxN.data;
			//			Si la estancia no es la actual, que ya comprobamos antes...
			if (auxEstancia.id != ((Mapa) mapas.search(mapaActual).data)
					.getEstanActual().id) {
				Item auxItem = buscaItemEst(auxEstancia, id);
				if (auxItem != null)
					return auxItem;
			}
			auxN = auxMapa.estancias.next();
		}
		return null;
	}

	/**
	 * Busca un Item en todos los mapa, cuyo identificador coincida con el que
	 * pasamos por parmetro
	 * 
	 * @param id
	 *            Identificador del Item a buscar.
	 * @return El item encontrado o null en caso de no encontrarlo.
	 */
	private Item buscaItemTM(int id) {
		Node auxN = mapas.root;
		Mapa auxMapa;
		mapas.inorden(auxN);
		auxN = mapas.next();
		while (auxN != null) {
			auxMapa = (Mapa) auxN.data;
			//			Si el mapa no es el actual, que ya comprobamos antes...
			if (auxMapa.id != ((Mapa) mapas.search(mapaActual).data).id) {
				Item auxItem = buscaItemM(auxMapa, id);
				if (auxItem != null)
					return auxItem;
			}
			auxN = mapas.next();
			;
		}
		return null;
	}

	/**
	 * Hace avanzar un paso a Oscar, en su camino hacia el destino.
	 */
	private void avanzarPaso() {
		int x = getPixelX(buscaRuta.getX(buscaRuta.path[pasoActual]));
		int y = getPixelY(buscaRuta.getY(buscaRuta.path[pasoActual]));
		if (y < oscar.getY() && direccion != 1) {
			//Cargamos y colocamos el Sprite de Oscar de espaldas.
			Utils.cargarImagen(Utils.spOscarEspa);
			oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			oscar.setFrameSequence(Utils.getFrames(Utils.spOscarEspa));
			direccion = 1;
		} else if (y > oscar.getY() && direccion != 0) {
			//Cargamos y colocamos el Sprite de Oscar de frente.
			Utils.cargarImagen(Utils.spOscarFren);
			oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			oscar.setFrameSequence(Utils.getFrames(Utils.spOscarFren));
			direccion = 0;
		} else if (x > oscar.getX() && direccion != 3) {
			//Cargamos y colocamos el Sprite de Oscar hacia el lado derecho.
			Utils.cargarImagen(Utils.spOscarLado);
			oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			oscar.setFrameSequence(Utils.getFrames(Utils.spOscarLado));
			oscar.setTransform(Sprite.TRANS_NONE);
			direccion = 3;
		} else if (x < oscar.getX() && direccion != 2) {
			//Cargamos y colocamos el Sprite de Oscar.
			Utils.cargarImagen(Utils.spOscarLado);
			oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			oscar.setFrameSequence(Utils.getFrames(Utils.spOscarLado));
			oscar.setTransform(Sprite.TRANS_MIRROR);
			direccion = 2;
		}
		oscar.setPosition(x, y);
		//Cada 4 celdas de camino, avanzo un sprite
		if (pasoActual % 4 == 0)
			oscar.nextFrame();
		redibujar = true;
		if (pasoActual < buscaRuta.path_count - 1)
			pasoActual++;
		else {
			caminando = false;
			ocupado = false;//en el caso de reCaminarBlo
			direccion = 0;
			Utils.cargarImagen(Utils.spOscarFren);
			oscar.setImage(Utils.imagen, Utils.anchoIm, Utils.altoIm);
			oscar.setFrameSequence(Utils.getFrames(Utils.spOscarFren));
		}
	}

	/**
	 * Indica, atendiendo al estado actual de la partida, si se puede guardar o
	 * no la partida.
	 * 
	 * @return Si se puede guardar o no la partida.
	 */
	public boolean puedeGuardar() {
		if (parteJuego != video && parteJuego != conversacion
				&& !mostrarMensaje)
			return true;
		return false;
	}

	/**
	 * Guarda en la memoria del mvil, la partida en curso.
	 * 
	 * @param g
	 *            objeto Graphics necesario para llamar a drawScreen y que de
	 *            esta manera se cargue la imagen de cargando
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public void guardarPartida(Graphics g) throws XmlPullParserException,
			IOException {
		//		cargamos el fondo de pantalla de guardando
		Utils.cargarImagen(Utils.imGuardando);
		sprite = null;
		sprite = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		sprite.setPosition(Utils.inicioPantallaX, Utils.inicioPantallaY);
		sprite.setFrameSequence(Utils.getFrames(Utils.imGuardando));
		sprite.paint(g);

		drawScreen(g);

		midlet.guardarDatos();
	}

	/**
	 * Inserta un registro en el RMS que est actualmente abierto.
	 * 
	 * @param baos
	 *            Stream donde est la informacin a guardar
	 * @throws RecordStoreNotOpenException
	 * @throws InvalidRecordIDException
	 * @throws RecordStoreFullException
	 * @throws RecordStoreException
	 */
	private void insertarRegistro(ByteArrayOutputStream baos, RecordStore rs)
			throws RecordStoreNotOpenException, InvalidRecordIDException,
			RecordStoreFullException, RecordStoreException {
		byte[] registro = baos.toByteArray();
		rs.addRecord(registro, 0, registro.length);
		baos.reset();
	}

	/**
	 * Establece a null todos los objetos del GameCanvas
	 */
	private void salir() {
		midlet = null;
		t = null;
		LM = null;
		parser = null;
		buscaRuta = null;
		mapas = null;
		escenActual = null;
		escenDetras = null;
		escenCaminable = null;
		sprite = null;
		oscar = null;
		puntero = null;
		itemsInventario = null;
		seleccionado = null;
		videosFase = null;
		converFase = null;
		participante = null;
		iniIdOp = null;
		resultados = null;
		ultimoEscenario = null;
	}

}
