
import java.util.Vector;
import javax.microedition.lcdui.game.Sprite;

/**
 * Entre 2 Almas	[https://sourceforge.net/projects/entre2almas/]  
 * ------------------------------------------------------------------------
 * This file is part of "Entre 2 Almas".
 * "Entre 2 Almas" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * "Entre 2 Almas" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "Entre 2 Almas"; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ------------------------------------------------------------------------
 * 		Copyright 2005 - 2006 Pedro Javier Sez Martnez
 * 
 * 
 * Un objeto Mapa, tendr objetos Estancia. Implementa la interfaz Comparable y
 * Opcionable.
 * 
 * @author Pedro Javier Sez Martnez
 * @version 1.0
 */
public class Estancia implements Opcionable, Comparable {

	/**
	 * Identificador de la Estancia
	 */
	int id;

	/**
	 * Escenarios disponibles para una determinada estancia
	 */
	public AVLTree escenarios;

	/**
	 * Nombre de la estancia
	 */
	String nombre;

	/**
	 * Indica el id del escenario activo actual.
	 */
	public int actual;

	/**
	 * Imagen que representar a la estancia en el mapa
	 */
	public Sprite representa;

	/**
	 * Posicin X en la que la imagen que representa esta estancia, se colocar
	 * en el mapa
	 */
	public int posX;

	/**
	 * Posicin Y en la que la imagen que representa esta estancia, se colocar
	 * en el mapa
	 */
	public int posY;

	/**
	 * Array que indica que opciones est o no disponibles
	 */
	public Opcion[] acciones;

	/**
	 * @param id
	 *            Identificador de la estancia.
	 * @param imagen
	 *            identificador de Utils que define la imagen que representa
	 *            esta estancia
	 * @param nombre
	 *            Nombre de la estancia
	 * @param actual
	 *            Escenario activo actualmente
	 * @param posX
	 *            posicin X del sprite de esta estancia en el mapa
	 * @param posY
	 *            posicin Y del sprite de esta estancia en el mapa
	 * @param acciones
	 *            Array que indica que opciones tendr o no disponibles el Item
	 * @param escenarios
	 *            Escenarios que forman parte de la Estancia.
	 * @param visible
	 *            Indica si la estancia ser visible en el mapa
	 */
	public Estancia(int id, String imagen, String nombre, int actual, int posX,
			int posY, Vector acciones, AVLTree escenarios, boolean visible) {
		this.id = id;
		this.escenarios = escenarios;
		this.nombre = nombre;
		this.actual = actual;
		if (Utils.grupo == 1) {//si el grupo es 176x200
			posX = (posX * Utils.aumentaX) / 1000;
			posY = (posY * Utils.aumentaY) / 1000;
		}
		this.posX = posX;
		this.posY = posY;
		//		Inicializamos a null las opciones disponibles.
		this.acciones = new Opcion[acciones.size()];
		acciones.copyInto(this.acciones);
		//		cargamos la representacin de la estancia
		Utils.cargarImagen(imagen);
		representa = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		representa.setPosition(Utils.inicioPantallaX + posX,
				Utils.inicioPantallaY + posY);
		representa.setFrameSequence(Utils.getFrames(imagen));
		this.representa.setVisible(visible);
	}

	/**
	 * Devuelve el escenario que actualmente esta activo en esta estancia
	 * 
	 * @return el escenario activo para una estancia
	 */
	public Escenario getEscenActual() {
		return (Escenario) escenarios.search(actual).data;
	}

	/**
	 * Indica si una determinada accin tiene o no alguna consecuencia
	 * 
	 * @param opcionSele
	 *            La opcin seleccionada en el juego
	 * @return boolean indicando si esa opcin o accin tiene alguna
	 *         consecuencia si es true es que no hay consecuencias, y viceversa.
	 */
	public boolean getConseActivaAcc(int opcionSele) {

		return acciones[opcionSele].consecuencias.isEmpty();
	}

	/**
	 * Establece la consecuencia activa para una determinada accin.
	 * 
	 * @param id
	 *            Identificador de la opcin a modificar.
	 * @param valor
	 *            el nuevo valor de conseActiva
	 */
	public void setConseActivaAcc(String id, int valor) {
		Opcion auxOp = getOpcion(id);
		auxOp.conseActiva = valor;
	}

	/**
	 * Devuelve la consecuencia para una determinada accin
	 * 
	 * @param opcionSele
	 *            La opcin seleccionada en el juego
	 * @return Vector con la consecuencia activa para una determinada accin
	 */
	public Vector getConsecuenciaAcc(int opcionSele) {

		return (Vector) acciones[opcionSele].consecuencias
				.elementAt(acciones[opcionSele].conseActiva);
	}

	/**
	 * El nombre de una determinada accin.
	 * 
	 * @param opcionSele
	 *            La opcin seleccionada en el juego
	 * @return nombre de la accin.
	 */
	public String getNombreAccion(int opcionSele) {
		return acciones[opcionSele].texto;
	}

	/**
	 * Devuelve el numero de acciones disponibles para un Seleccionable
	 * 
	 * @return el numero de acciones disponibles para un Seleccionable
	 */
	public int longAcciones() {
		return acciones.length;
	}

	/**
	 * Establece si una accin esta activa o no
	 * 
	 * @param opcionSele
	 *            La opcin seleccionada en el juego
	 * @param valor
	 *            boolean indicando si est activa o no.
	 */
	public void setAccionActiva(int opcionSele, boolean valor) {
		acciones[opcionSele].activa = valor;
	}

	/**
	 * Indica si una determinada accin esta activa o no.
	 * 
	 * @param opcionSele
	 *            La opcin seleccionada en el juego
	 * @return boolean indicando si la opcin est o no activa.
	 */
	public boolean getAccionActiva(int opcionSele) {
		return acciones[opcionSele].activa;
	}

	/**
	 * Devuelve la clave del comparable
	 * 
	 * @return La clave del comparable identificador del comparable.
	 */
	public int getKey() {
		return id;
	}

	/**
	 * Devuelve la opcin cuyo id coincide con el que se pasa como parmetro
	 * 
	 * @param id
	 *            String con el identificador de la opcin
	 * @return La opcin cuyo id coincide con el de la opcin.
	 */
	public Opcion getOpcion(String id) {
		for (int i = 0; i < longAcciones(); i++) {
			if (acciones[i] != null) {
				if (acciones[i].id.equals(id)) {
					return acciones[i];
				}
			}
		}
		return null;
	}
}
