
import javax.microedition.lcdui.game.Sprite;

/**
 * Entre 2 Almas	[https://sourceforge.net/projects/entre2almas/]  
 * ------------------------------------------------------------------------
 * This file is part of "Entre 2 Almas".
 * "Entre 2 Almas" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * "Entre 2 Almas" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "Entre 2 Almas"; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ------------------------------------------------------------------------
 * 		Copyright 2005 - 2006 Pedro Javier Sez Martnez
 * 
 * 
 * El juego tendr al menos un objeto de tipo Mapa. Implementa la interfaz
 * Comparable.
 * 
 * @author Pedro Javier Sez Martnez
 * @version 1.0
 */
public class Mapa implements Comparable {

	/**
	 * Identificador del Mapa
	 */
	int id;

	/**
	 * Arbol AVL con las estancias activas del mapa
	 */
	public AVLTree estancias;

	/**
	 * Variable que indica el id de la estancia est activa
	 */
	public int activa;

	/**
	 * Imagen de fondo del mapa
	 */
	Sprite representa;

	/**
	 * Constructor del Mapa
	 * 
	 * @param id
	 *            identificador del mapa.
	 * @param representa
	 *            Imagen de fondo del mapa
	 * @param estancias
	 *            Estancias activas del mapa
	 * @param activa
	 *            Variable que indica cual de las estancias est activa
	 */
	public Mapa(int id, String representa, AVLTree estancias, int activa) {
		this.id = id;
		this.estancias = estancias;
		this.activa = activa;
		//cargamos la imagen de fondo del mapa
		Utils.cargarImagen(representa);
		this.representa = new Sprite(Utils.imagen, Utils.anchoIm, Utils.altoIm);
		this.representa.setPosition(Utils.inicioPantallaX,
				Utils.inicioPantallaY);
		this.representa.setFrameSequence(Utils.getFrames(representa));
	}

	/**
	 * Devuelve la estancia actual
	 * 
	 * @return la estancia que actualmente tiene que ser mostrado
	 */
	public Estancia getEstanActual() {
		return (Estancia) estancias.search(activa).data;
	}

	/**
	 * Devuelve el escenario a mostrar
	 * 
	 * @return el escenario que actualmente tiene que ser mostrado
	 */
	public Escenario getEscenActual() {
		return this.getEstanActual().getEscenActual();
	}

	/**
	 * Devuelve la clave del comparable
	 * 
	 * @return La clave del comparable identificador del comparable.
	 */
	public int getKey() {
		return id;
	}

}
