// [ -------------------------------------------------- ]
//[ A * - A l g o r i t h m f o r J 2 M E ]
//[ -------------------------------------------------- ]
//
//[ -------------------------------------------------- ]
//[ code written by Sebastian Lehmann ]
//[ This is a simple version of the A*-Algorithm! ]
//[ -------------------------------------------------- ]
//[ note that my english is not the very best ]
//[ because I'm from Germany :-) ]
//[ -------------------------------------------------- ]
//
//IMPORTANT NOTES:
//While in other A* codes the coordinates of any point
//is given as a X and Y coordinate here there is the Y
//coordinate multiplied with the width and added to the
//X coordinate. This way decreases the use of memory which
//is in most cases a problem on mobile phones!
//Use functions getIndex(x,y), getX(index) and getY(index)
//to get the coordinates whether as an index value (for
//the use of array index) or as X / Y coords (for the use
//to calculate with coords).
//
//This algrorithm doesn't exactly equal A* because I
//don't use the "closed list". But that would only incre-
//ase the use of memory... I think that my way - without
//using closed list - is a bit slower in the speed of
//finding paths. But the memory is - as said - a big
//problem by using A* on mobile applications, so there
//is no other way...


public class PathFinding{
	// maximum map matrix size (needed to initialize arrays)
	int mapWidthMax  = 88; //     I M P O R T A N T ! ! !
	int mapHeightMax = 100; //     Change to max value of map size!!!
	
	// real map matrix size (needed to calculate coords and paths)
	int mapWidth; //the real size of the map
	int mapHeight;
	
	// the maximum index value of any array (used for FOR-LOOPS)
	int imax = mapWidthMax*mapHeightMax/8;
	// array to store whether a point is walkable or not
	// NOTE: set values from outside this class or add a function in this class to
	// do this job!
	boolean walkability[] = new boolean[mapWidthMax*mapHeightMax];
	// array to store all paths which are possibly the shortest (used during the
	// calculation)
	// You may ask: Why is the size (mapwidth*mapheight/8)??? => the count of
	// possible paths accepted (because of memory problems on mobile phones...)
	// IMPORTANT: an array field value represents the time needed to walk along the
	// path!
	// NOTE: THIS IS USUALLY CALLED "OPEN LIST" IN ANOTHER A*-CODES!
	int paths[] = new int[imax];
	
	// the currently count of stored paths in the array paths[] (used during the
	// calculation)
	// You may ask: Why is the count initialized with 1? => this path is the path
	// with the only field equalizing the START field
	int paths_count = 1;
	
	// the index of paths[] which represents the currently shortest path (the
	// shortest time) (used during the calculation)
	int shortest_path = 0;
	
	// the coordinate (index-based) of the last field of any PATH (!) (used during
	// the calculation)
	int paths_coord[] = new int[imax];
	
	// the direction of the parent field of any FIELD (!) (used during the
	// calculation)
	// IF -1: THIS FIELD HAS NO PARENT FIELD (ISN'T INCLUDED IN ANY PATH)
	byte parent[] = new byte[mapWidthMax*mapHeightMax];
	byte direction_up = 1;
	byte direction_down = 64;
	byte direction_left = 4;
	byte direction_right = 16;
	
	// the guessed way (time) of any path (used during the calculation)
	int paths_guessed_way[] = new int[imax];
	
	// used to exit the loop when the path was found
	boolean path_found = false;
	
	// the returned array containing the shortest path
	int path[] = new int[imax];
	// the count:
	int path_count;
	// time needed to walk along the whole path:
	int path_total_length = 0;
	
	//OPTIONAL: try different heuristic methods:
	//manhattan: is faster, but gives you not inevery ase the shortest path
	//real: takes more time but gives you in EVERY CASE the shortest path (if there
	// is one...)
	int manhattan = 1;
	int real = 2;
	int real_diagonalvalue = 14;  //if 14, then "real" finds as said the shortest
								  // path.
	                            //if 20, then "real" equals "manhattan" heuristic.
	                            //if between, then it's a userdefined heuristic.
	int heuristic = manhattan;
	
	
	public PathFinding(int width, int height){
		// the values width and height represent the size of the map matrix
		mapWidth = width;
		mapHeight = height;
	}
	
	public boolean findPath(int startx, int starty, int targetx, int targety){
		// this function is the main part:
		// it calculates the shortest (fastest) path to walk from START to TARGET
		// STARTX / STARTY: the start coordinates
		// TARGETX / TARGETY: the target (end) coordinates
		
		// RETURN VALUE: TRUE, IF THERE IS A POSSIBLE PATH; OTHERWISE FALSE.
		
		
		if (startx  < 0 || starty  < 0 || startx  > mapWidth-1 || starty  > mapHeight-1 ||
		  targetx < 0 || targety < 0 || targetx > mapWidth-1 || targety > mapHeight-1)
		  return false;
		
		
		// RESET THE VARIABLES
		path_found = false;
		    // clear parent array (fill it with byte values -1)
		for (int i1 = 0; i1 <= mapWidth*mapHeight-1; i1++)
		  parent[i1] = (byte)-1;
		// init the paths: the 1st path (and only path) is the path with the only field
		// equalizing the START field
		paths_count = 1;
		// it is the shortest path (because it's the only one...)
		shortest_path = 0;
		// the distance between START and this path is of course zero:
		paths[shortest_path] = 0;
		// now set the START coord to the array field:
		paths_coord[shortest_path] = getIndex(startx,starty);
		// its parents is itself:
		parent[paths_coord[shortest_path]] = (byte)0;
		
		
		// calculate the guessed way from the START to TARGET: (CALLD "H COST" IN
		// ANOTHER A*-CODES)
		int xoffset = Math.abs(getX(paths_coord[0]) - targetx);
		int yoffset = Math.abs(getY(paths_coord[0]) - targety);
		 
		if (heuristic == manhattan){
		  //MANHATTAN DISTANCE (ONLY NON-DIAGONAL:
		  paths_guessed_way[0] = (xoffset+yoffset)*10;
		}
		if (heuristic == real){
		  //REAL DISTANCE (DIAGONAL & NON-DIAGONAL):
		  if (xoffset > yoffset)
		    paths_guessed_way[0] = (xoffset-yoffset)*10 + yoffset*real_diagonalvalue;
		  else
		    paths_guessed_way[0] = (yoffset-xoffset)*10 + xoffset*real_diagonalvalue;
		}
		paths_guessed_way[0] += paths[0];
		
		
		// S T A R T S E A R C H I N G F O R T H E S H O R T E S T P A T H :
		
		
		// Loop the search while the end of the shortest path doesn't equal TARGET.
		// If there isn't any possible way to walk, the loop whil be exited.
		while (!path_found){
		
		  /////////////////////
		  //   S T E P 1 //
		  /////////////////////
		
		  //=> CALCULATE THE GUESSED WAY OF ANY PATH (CALLD "F COST" IN ANOTHER
		  // A*-CODES)
		  /*
		   * for (int i1 = 0; i1 <= paths_count-1; i1++) { // calculate the guessed way
		   * from the end of any path to TARGET: (CALLD "H COST" IN ANOTHER A*-CODES)
		   * xoffset = Math.abs(getX(paths_coord[i1]) - targetx); yoffset =
		   * Math.abs(getY(paths_coord[i1]) - targety); if (heuristic == manhattan) {
		   * //MANHATTAN DISTANCE (ONLY NON-DIAGONAL: paths_guessed_way[i1] =
		   * (xoffset+yoffset)*10; } if (heuristic == real) { //REAL DISTANCE (DIAGONAL &
		   * NON-DIAGONAL): if (xoffset > yoffset) paths_guessed_way[i1] =
		   * (xoffset-yoffset)*10 + yoffset*real_diagonalvalue; else
		   * paths_guessed_way[i1] = (yoffset-xoffset)*10 + xoffset*real_diagonalvalue; }
		   *  // and add to this way the way from START to the end of any path: (CALLD
		   * "G COST" IN ANOTHER A*-CODES)
		   * 
		   * paths_guessed_way[i1] += paths[i1]; }
		   */
		
		
		  /////////////////////
		  //   S T E P 2 //
		  /////////////////////
		
		  //=> FIND OUT THE CURRENTLY SHORTEST PATH ("LOWEST F SCORE" IN ANOTHER
		  // A*-CODES)
		
		  shortest_path = 0;
		 
		  for (int i1 = 0; i1 <= paths_count-1; i1++){
		    // is this way (-> i1) shorter than the shortest path so replace it:
		    if (paths_guessed_way[i1] < paths_guessed_way[shortest_path])
		      shortest_path = i1;
		  }
		
		
		
		  /////////////////////
		  //   S T E P 3 //
		  /////////////////////
		
		  //=> SEARCH FOR NEW PATHS AND ADD THEM TO THE ARRAYS ("OPEN LIST" IN ANOTHER
		  // A*-CODES)
		
		 
		  // try if the current field is positioned at the border of the map,
		  // and if true then only try the fields inside the map -
		  // otherwise there would be thrown a ArrayIndexOutOfBoundsExpression!
		  int i1min; int i1max; int i2min; int i2max;
		  if (getX(paths_coord[shortest_path]) <= 0)
		    i1min = 0;
		  else
		    i1min = -1;
		  if (getX(paths_coord[shortest_path]) >= mapWidth-1)
		    i1max = 0;
		  else
		    i1max = 1;
		  if (getY(paths_coord[shortest_path]) <= 0)
		    i2min = 0;
		  else
		    i2min = -1;
		  if (getY(paths_coord[shortest_path]) >= mapHeight-1)
		    i2max = 0;
		  else
		    i2max = 1;
		
		  // this for-loops try any field around the end of the shortest path
		  for (int i1 = i1min; i1 <= i1max; i1++){ // XOFFSET
		    for (int i2 = i2min; i2 <= i2max; i2++){ // YOFFSET
		      int thisx = getX(paths_coord[shortest_path])+i1;
		      int thisy = getY(paths_coord[shortest_path])+i2;
		
		      // if the field is neither itself nor START nor unwalkable ...
		      if ( (!(i1 == 0 && i2 == 0)) && !(getIndex(thisx,thisy) == getIndex(startx,starty)) && walkability[getIndex(thisx,thisy)]) {
		        // calc time needed to walk from old to new field:
		        int t_needed = 0;
		        if (i1 != 0)
		          t_needed += 10;
		        if (i2 != 0)
		          t_needed += 10;
		        if (t_needed == 20)
		          t_needed = 14;
		
		
		        // if this field is already included in another path (IS IN OPEN OR
				// CLOSED LIST),
		        // then check which path is the shorter one:
		        if (parent[getIndex(thisx,thisy)] > 0) {
		         
		          // what's the path# which already exists? => i3
		          int i3;
		          boolean not_on_closed_list = false;
		          search:
		          for (i3 = 0; i3 <= paths_count-1; i3++){
		            if (paths_coord[i3] == getIndex(thisx,thisy)){
		              not_on_closed_list = true;
		              break search;
		            }
		          }
		
		          //if it isn't on closed list then replace the old field with the new
				  // one:
		          if (not_on_closed_list){
		            if (paths[shortest_path] + t_needed < paths[i3]) {
		              // set its length (time needed to walk) (G COST)
		              paths[i3] = paths[shortest_path] + t_needed;
		
		              // set the new guessed way (F COST)
		              xoffset = Math.abs(thisx - targetx);
		              yoffset = Math.abs(thisy - targety);
		             
		              if (heuristic == manhattan){
		                //MANHATTAN DISTANCE (ONLY NON-DIAGONAL:
		                paths_guessed_way[i3] = (xoffset+yoffset)*10;
		              }      
		              if (heuristic == real) {
		                //REAL DISTANCE (DIAGONAL & NON-DIAGONAL):
		                if (xoffset > yoffset)
		                  paths_guessed_way[i3] = (xoffset-yoffset)*10 + yoffset*real_diagonalvalue;
		                else
		                  paths_guessed_way[i3] = (yoffset-xoffset)*10 + xoffset*real_diagonalvalue;
		              }
		              paths_guessed_way[i3] += paths[i3];
		
		
		              // set the parent direction of the new field
		              parent[paths_coord[i3]] = 0;
		              if (i1==1)
		                parent[paths_coord[i3]] += direction_left;
		              if (i1==-1)
		                parent[paths_coord[i3]] += direction_right;
		              if (i2==1)
		                parent[paths_coord[i3]] += direction_up;
		              if (i2==-1)
		                parent[paths_coord[i3]] += direction_down;
		            }
		          }
		        }
		        else {// if it's not already on open list, then add it:
		          // add it to the arrays (OPEN LIST)
		          paths_coord[paths_count] = getIndex(thisx,thisy);
		
		
		          // set its length (time needed to walk) (G COST)
		          paths[paths_count] = paths[shortest_path] + t_needed;
		
		
		          // set its guessed way (F COST)
		          xoffset = Math.abs(thisx - targetx);
		          yoffset = Math.abs(thisy - targety);
		          if (heuristic == manhattan){
		            //MANHATTAN DISTANCE (ONLY NON-DIAGONAL:
		            paths_guessed_way[paths_count] = (xoffset+yoffset)*10;
		          }      
		          if (heuristic == real) {
		            //REAL DISTANCE (DIAGONAL & NON-DIAGONAL):
		            if (xoffset > yoffset)
		              paths_guessed_way[paths_count] = (xoffset-yoffset)*10 + yoffset*real_diagonalvalue;
		            else
		              paths_guessed_way[paths_count] = (yoffset-xoffset)*10 + xoffset*real_diagonalvalue;
		          }
		
		          paths_guessed_way[paths_count] += paths[paths_count];
		
		          // set the parent direction of the new field
		          parent[paths_coord[paths_count]] = 0;
		          if (i1==1)
		            parent[paths_coord[paths_count]] += direction_left;
		          if (i1==-1)
		            parent[paths_coord[paths_count]] += direction_right;
		          if (i2==1)
		            parent[paths_coord[paths_count]] += direction_up;
		          if (i2==-1)
		            parent[paths_coord[paths_count]] += direction_down;
		
		
		          // increase the count of paths in the open list
		          paths_count++;
		        }
		      }
		    }
		  }
		
		
		
		
		
		
		
		  /////////////////////
		  //   S T E P 5 //
		  /////////////////////
		
		  //=> DELETE THE PATH FROM OPEN LIST
		
		  if (!path_found) {
		     
		    // IF THIS PATH HAS REACHED THE TARGET THEN EXIT LOOP
		    if (paths_coord[shortest_path] == getIndex(targetx,targety)) {
		      path_found = true;
		      //zeit fALr den weg
		      path_total_length = paths[shortest_path];
		    }
		
		    // CAUTION: IF THE PATH IS THE ONY ONE => THEN THERE IS NO POSSIBLE PATH !!!
			// => EXIT SEARCH
		    if (paths_count == 1)
		      break;
		
		    // delete the old shortest path from the OPEN LIST:
		    for (int i3 = shortest_path+1; i3 <= paths_count-1; i3++) {
		      //shift any entry (entry# = i3) to the index before:
		      paths[i3-1] = paths[i3];
		      paths_coord[i3-1] = paths_coord[i3];
		      paths_guessed_way[i3-1] = paths_guessed_way[i3];
		    }
		    paths_count--;
		  }
		
		}
		
		if (!path_found)
		  return (false);
		else{
		   
		  //  AFTER THE SEARCH:
		
		  // follow the parent's history backwards
		
		  int fx = targetx;  // the backwards followed coordinates
		  int fy = targety;
		  int n = 0;  // the number of backwards followed points
		
		
		
		  // DIESE WHILE-SCHLEIFE STELLT DIE RAsCKVERFOLGUNG DES PFADES DAR.
		  // MANCHMAL ERGIBT DIE WHILE-SCHLEIFE EINE ENDLOSSCHLEIFE!
		  // IN DIESEM FALL EINFACH AUSKOMMENTIEREN ( /*...*/ )
		
		
		  //loop until the path follow reached START
		  while(!(fx == startx && fy == starty))  {
		    if     (parent[getIndex(fx,fy)] == direction_left                   )
		    { addPoint(n,fx,fy); fx--;       }
		    else if(parent[getIndex(fx,fy)] == direction_left  + direction_up   )
		    { addPoint(n,fx,fy); fx--; fy--; }
		    else if(parent[getIndex(fx,fy)] ==                   direction_up   )
		    { addPoint(n,fx,fy);       fy--; }
		    else if(parent[getIndex(fx,fy)] == direction_right + direction_up   )
		    { addPoint(n,fx,fy); fx++; fy--; }
		    else if(parent[getIndex(fx,fy)] == direction_right                  )
		    { addPoint(n,fx,fy); fx++;       }
		    else if(parent[getIndex(fx,fy)] == direction_right + direction_down )
		    { addPoint(n,fx,fy); fx++; fy++; }
		    else if(parent[getIndex(fx,fy)] ==                   direction_down )
		    { addPoint(n,fx,fy);       fy++; }
		    else if(parent[getIndex(fx,fy)] == direction_left  + direction_down )
		    { addPoint(n,fx,fy); fx--; fy++; }
		    n++;
		  }
		
		
		
		  //path[] nach vorne "holen"
		  for (int i1 = 0; i1 <= n-1; i1++){
		    path[i1] = path[imax - n + i1];
		  }
		
		  //anzahl pfadpunkte
		  path_count = n;
		 
		  return (true);
		
		}
	}
	
	
	int getIndex(int x, int y){
		return (y * mapWidth + x);
	}
	
	int getX(int index){
		return (index % mapWidth);
	}
	
	int getY(int index){
		return ((index - getX(index)) / mapWidth);
	}
	
	
	void addPoint(int n, int x, int y){
	
		path[imax - n - 1] = getIndex(x,y);
	
		// to find errors, uncomment this line to list the path points
		//System.out.println("x:" + x + " y:" + y + " direction:" +
		// parent[getIndex(x,y)]);
	}
}
